<?php

namespace install\lib;

/*
 * aribus systems - Version 7.0.0
 * 
 * Copyright (C) 2018  Sascha Momcilovic
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 3 of the License, or (at your option) 
 * any later version. This program is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, see <http://www.gnu.org/licenses/>. 
 * 

 *
 * @author: Sascha Momcilovic
 * @e-Mail: info@aribu.systems
 * @link:   https://aribu.systems
 * @date:   01.06.2018
 */

class Installation {
    
    public function __construct($TT, $http, $be_language) {
        $this->TT = $TT;
        $this->http = $http;
        $this->prepareLanguage($be_language);
        
        if($this->http->issetPost('SaveSettings') === false) {
            $this->PreInstallation();
        } else {
            $this->SaveSettings();
        }
    }
    
        
    public function PrepareLanguage($be_language) {
        if(is_array($be_language)) {
            foreach($be_language as $key => $value) {
                $this->TT->setMC($key, $value);
            }
        }
    }
    
    protected function PreInstallation() {
        $this->TT->setMC('database_port',   '3306')
                 ->setMC('database_host',   'localhost')
                 ->setMC('api_key',         md5($_SERVER['HTTP_HOST']))
                 ->setMC('http',            'http://'.$_SERVER['HTTP_HOST'].'/')
                 ->setMC('httpo',           'http://'.$_SERVER['HTTP_HOST'])
                 ->setMC('https',           'https://'.$_SERVER['HTTP_HOST'])
                 ->setMC('url_host',        $_SERVER['HTTP_HOST']);
    }
    
    protected function SaveSettings() {
        $vars = $this->http->_post('sysconfig');
        
        define('DB_FUNCTION', DOC_ROOT . '/lib/database/pdo/class.dbfunctions.php');
                
        # Database constans
        define('DB_USER',               $vars['database_username']);
        define('DB_PASSWORD',           $vars['database_password']);
        define('DB_NAME',               $vars['database_name']);
        define('DB_HOST',               $vars['database_host']);
        define('DB_PORT',               intval($vars['database_port']));
        define('TABLE_PRE_NAME',        'fe_');
        
        define('ARIBU_API_PASSWORD',    $vars['api_password']);
        
        require_once(DB_FUNCTION);
        
        # Import SQL
        $db->Import(DOC_ROOT.'/install/aribu-systems-db.sql');
        
        # Update User
        $password = md5($this->http->_post('password1'));
        $EncryptedPassword = \lib\vendor\defuse\crypto\Crypto::encryptWithPassword($this->http->_post('password1'), ARIBU_API_PASSWORD);
        $gruppen = $this->http->_post('be_group') != 'x' ? implode("-", $this->http->_post('be_group')) : "";
        
        $db->TablePreName = "be_";
        $db->Update('user')
           ->WhereColName('uid')
           ->WhereColValue(1) 
           ->Set(array(   "username"          => $this->http->_post('username'),
                          "password"          => $password,
                          "admin"             => 1,
                          "disable"           => 0,
                          "email"             => $this->http->_post('email'),
                          "full_name"         => $this->http->_post('full_name'),
                          "userCache"         => $EncryptedPassword,
                          "lang"              => $GLOBALS['LANGUAGE'],
                          "usergroup"         => $gruppen,
                          "author"            => $this->http->_post('author'),
                          "user_image"        => '',
                          "fe_user_rights"    => 1024))
           ->doUpdate();
        
        # Update client
        $db->TablePreName = "fe_";
        $db->Update('client')
           ->WhereColName('client_id')
           ->WhereColValue(1)
           ->Set(array( "client_name"               => $this->http->_post('client_name'),
                        "status"                    => 1,
                        "http"                      => $this->http->_post('http'),
                        "httpo"                     => $this->http->_post('httpo'),
                        "https"                     => $this->http->_post('https'),
                        "url_host"                  => $this->http->_post('url_host'),
                        "url_protocol"              => $this->http->_post('url_protocol'),
                        "url_port"                  => $this->http->_post('url_port'),
                        "client_email_address"      => $this->http->_post('client_email_address'),
                        "client_email_name"         => $this->http->_post('client_email_name'),
                        "browser_name"              => $this->http->_post('browser_name'),
                        "client"                    => $this->http->_post('client')))
           ->doUpdate();
        
        # Write sys.conf.php 
        $sysconf = \lib\Functions::readfiles('/sys/sys.conf.inc');
        $sysconf .= "   \n\n# Database constans
define('DB_USER',               '".$vars['database_username']."');
define('DB_PASSWORD',           '".$vars['database_password']."');
define('DB_NAME',               '".$vars['database_name']."');
define('DB_HOST',               '".$vars['database_host']."');
define('DB_PORT',               '".intval($vars['database_port'])."');
define('TABLE_PRE_NAME',        'fe_');

# Language
define('SYS_LANGUAGE',          '".$GLOBALS['LANGUAGE']."');

# Timezone
define('TIMEZONE',              'Europe/Berlin');

# Api
define('ARIBU_API_KEY',         '".$vars['api_key']."');
define('ARIBU_API_PASSWORD',    '".$vars['api_password']."');";
        
        # Neues Template Speichern
        $filename = DOC_ROOT . "/sys/sys.conf.php";

        # Der Dateizeiger befindet sich am Anfang der Datei, und
        # dort wird $this->http->_post('temp_body') später mit fwrite() geschrieben.
        if (!$handle = fopen($filename, "w")) {
            $this->Error['Validation'] = str_replace('###file###', $filename, $this->TT->getMC('_read_error'));         
        }

        # Schreibe $this->http->_post('temp_body') in die ge&ouml;ffnete Datei.
        $tmp = stripslashes($sysconf);
        if (!fwrite($handle, $tmp)) {
            $this->Error['Validation'] .= str_replace('###file###', $filename, $this->TT->getMC('_write_error'));  
        }
        fclose($handle);
        
        unlink(DOC_ROOT."/sys/sys.conf.inc");
        
        
        # Copy plugin templates to client
        try { 
            if(\lib\Functions::CheckIfDirectoryExists('/_templates/'.$this->http->_post('client_name'))) {}
            \lib\Functions::rcopy(DOC_ROOT.'/_templates/plugins', DOC_ROOT.'/_templates/'.str_replace(':'.$this->http->_post('url_port'), '', $this->http->_post('client_name')).'/plugins');

            # Copy elements templates to client
            \lib\Functions::rcopy(DOC_ROOT.'/_templates/elements', DOC_ROOT.'/_templates/'.str_replace(':'.$this->http->_post('url_port'), '', $this->http->_post('client_name')).'/elements');

            # Copy stylesheets
            \lib\Functions::rcopy(DOC_ROOT.'/fileadmin/css', DOC_ROOT.'/fileadmin/css/'.str_replace(':'.$this->http->_post('url_port'), '', $this->http->_post('client_name')));
        } catch (\Exception $exc) {
            $this->Error['Validation'] = $exc->getTraceAsString();
        }
        
        # Ready to rumble
        $this->TT->setMC('successfull_installation',    $this->TT->getMC('_successfull_installation').'<br>Installations-Meldungen: '.$this->Error['Validation'])
                 ->setMC('link_to_admin_panel',         $this->http->_post('url_protocol').'://'.$this->http->_post('url_host').'/aribu.sys/');
        
        $this->TT->BlockMarker['installed'] = 0;
        
    }
    
} 