<?php

namespace lib;

/*
 * aribus systems - Version 7.0.0
 * 
 * Copyright (C) 2018  Sascha Momcilovic
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 3 of the License, or (at your option) 
 * any later version. This program is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, see <http://www.gnu.org/licenses/>. 
 * 

 *
 * @author: Sascha Momcilovic
 * @e-Mail: info@aribu.systems
 * @link:   https://aribu.systems
 * @date:   20.02.2006
 * @update: 04.04.2016
 */

class Frontend extends \lib\objects\Loader {

    public $id                      = "";       # the page id
    public $spid                    = 1;        # Start page ID
    public $QueryString;
    public $RequestURI;
    public $pagename                = "";       # the pagename
    public $page_Hash               = null;     # Hash of the pagetree
    public $PageTreePoints;
    public $PageTreePointsHashes;
    public $PageTreePointsLevel;
    public $TemplateAssignArray     = array();
    public $TemplatePage            = "";       # this is the TemplatePage
    public $PHPTag                  = 0;        # For own PHP-Scripts
    public $PHP_Path                = "";       # Holds the path of the own PHP-Script
    public $PHPMarkerPath           = array();
    public $HTMLOldHeader           = "";       # Holds the Old HTML Header
    public $CFromPID;                           # Content from pid
    public $AdminShowHidden         = 0;        # Admin preview	pages
    public $RootFolder;
    public $Counter                 = array();
    public $LastPointer             = 0;
    public $ColPosition;
    public $PageCache               = 0;
    public $PageExpireCache;
    public $CachedContent           = "";
    public $PageCacheUpdate         = false;
    public $Error                   = 0;
    public $ChieldBuffer            = "";
    public $Description             = "";
    public $BrowserTitle            = "";
    public $ElementContent          = "";
    public $ElementMarker           = "";
    public $IndexFollow             = 1;
    public $FID;
    public $TID;
    public $MSID;
    public $URLHash;
    public $AdminShowContent        = 0;
    public $Index                   = 0;
    public $PageConfig              = "";
    public $JavaScript              = "";
    public $Stylesheet              = "";
    public $Pages                   = "pages";
    public $and_lang                = "";
    public $PageSettings            = "";
    public $jumpURL;
    public $ContentID;
    public $CTag = '';
        
    /*
     * Frontend construct
     * 		
     */

    public function __construct() {
        parent::__construct();
         
        if ($this->http->issetPost('id')) {
            $this->id = intval($this->http->_post('id'));
            $GLOBALS['LANGUAGE'] = !isset($GLOBALS['LANGUAGE']) ? SYS_LANGUAGE : $GLOBALS['LANGUAGE'];
        } else {
            # Request Uri
            $this->RequestURI = $this->http->_RequestUri();
            
            # Query String
            $this->QueryString = $_SERVER['QUERY_STRING'];
            
            # Set language
            $this->setLanguage();
            
             # Set hash
            $this->setHash();
            
            # Set page hash
            $this->setPageHash();
        }

        $this->setDBLanguageTable();
        $this->db->TablePreName = 'fe_';
    }
    
    
    /*
     * Set the global language
     * 
     */
    private function setLanguage() {
        $GLOBALS['LANGUAGE'] = SYS_LANGUAGE;
        $uriArr = Functions::pagenameExplode("/", $this->RequestURI);
        if (count($uriArr) > 2) {
            if (strlen($uriArr[1]) == 2 && is_string($uriArr[1])) {
                $GLOBALS['LANGUAGE'] = \lib\Functions::removeCriticSigns($uriArr[1]);
            } else {
                $this->RootFolder = $uriArr[1];
                $get_id = intval($uriArr[2]);
                if ($get_id != 0) {
                    $GLOBALS['GET_ID'] = $uriArr[2];
                }
            }
        } 
    }
    
    
    /*
     * Set page hash
     */
    private function setPageHash() {
        
        $this->db->Select('*')
                 ->From($this->Pages)
                 ->WhereColName('nav_title')
                 ->WhereColValue(URL_HOST)
                 ->Andd(str_replace("%%%n%%%", "", $this->and_lang) 
                        . " AND docType = 0 "
                        . " AND ((starttime = 0 AND endtime = 0 AND hidden != '1') "
                         . "OR (starttime != 0 AND starttime <= " . time() . " AND endtime = 0 AND hidden != '1') "
                         . "OR (starttime != 0 AND starttime <= " . time() . " AND endtime != 0 AND endtime >= " . time() . " AND hidden != '1') "
                         . "OR (starttime = 0 AND endtime != 0 AND endtime >= " . time() . " AND hidden != '1') "
                         . "OR (starttime != 0 AND starttime >= " . time() . " AND endtime = 0 AND hidden = '1') "
                         . "OR (starttime != 0 AND starttime >= " . time() . " AND endtime != 0 AND endtime <= " . time() . " AND hidden = '1') "
                         . "OR (starttime = 0 AND endtime != 0 AND endtime <= " . time() . " AND hidden = '1') "
                         . "OR (starttime != 0 AND starttime < " . time() . " AND endtime != 0 AND endtime <= " . time() . " AND hidden = '1') "
                         . "OR (starttime != 0 AND starttime >= " . time() . " AND endtime != 0 AND endtime > " . time() . " AND hidden = '1'))")
                 ->Parameter($this->DFE->and_lang != '' ? array('lang' => $GLOBALS['LANGUAGE']) : array())        
                 ->doSelect();
        $num = $this->db->Rows;
        
        $pageHash_prefix = $num == 1 ? '/'.URL_HOST : '';
        
        if (!preg_match('/.html/ism', $this->RequestURI)) {
            $file_doc = ".html";
        }
        
        if ($GLOBALS['LANGUAGE'] != SYS_LANGUAGE) {
            $this->RUO = substr($this->RequestURI . $file_doc, 3);
            $this->page_Hash = $this->RequestURI != '/' ? md5($pageHash_prefix.substr($this->RequestURI . $file_doc, 3)) : ($GLOBALS['CLIENT'] == 0 ? '' : md5(URL_HOST) );
        } else {
            $this->RUO = $this->RequestURI . $file_doc;
            $this->page_Hash = $this->RequestURI != '/' ? md5($pageHash_prefix.$this->RequestURI . $file_doc) : ($GLOBALS['CLIENT'] == 0 ? '' : md5(URL_HOST) );
        }
    }
    
    
    /*
     * Set hash
     */
    private function setHash() {
        
        # ID's Update
        $url_seperator = isset($GLOBALS['URL']['Seperator']) ? $GLOBALS['URL']['Seperator'] : "/b/";
            
        if (preg_match('%'.$url_seperator.'%ism', $this->RequestURI)) {
            
            $req_url = explode($url_seperator, $this->RequestURI);
            
            $this->RequestURI = $req_url[0] == '' ? '/' : $req_url[0];
            $hash = strstr($req_url[1], "/") ? explode("/", $req_url[1]) : str_replace(".html", "", $req_url[1]);

            if (is_array($hash)) {
                switch (count($hash)) {
                    case 2:
                        $_POST['SEITE'] = $hash[0];
                        $this->http->setPostbyName('SEITE', $hash[0]);
                    break;
                    case 3:
                        $_POST['SEITE'] = $hash[0];
                        $_POST['Sorting'] = !isset($_POST['Sorting']) ? $hash[1] : $_POST['Sorting'];
                        $this->http->setPostbyName('SEITE', $hash[0]);
                        $this->http->setPostbyName('Sorting', !isset($_POST['Sorting']) ? $hash[1] : $_POST['Sorting']);
                    break;
                }
                $hash = end($hash);
            } else {
                $hash = $hash;
            }
            $this->FID = !empty($hash) ? md5($hash) : "";
            $this->URLHash = $hash;
            
            
        } 
        
        # Pagename
        $pagename = explode("/", $this->RequestURI);
        $this->pagename = end($pagename);
    }
    
    
    /*
     * Sets the database to its language table
     */
    private function setDBLanguageTable() {
        if ($GLOBALS['LANGUAGE'] != SYS_LANGUAGE) {
            $this->Pages = "pages_language";
            $this->and_lang = "AND %%%n%%%lang = :lang ";
        } else {
            $this->Pages = "pages";
            $this->and_lang = "";
        }
    }

    /*
     * Page Globals
     * 		
     */

    public function PageGlobals() {

        # Set locale
        setlocale(LC_ALL, $GLOBALS['SYSTEM_LOCALE'][$GLOBALS['LANGUAGE']]);
        $this->db->TablePreName = 'fe_';
        if (!empty($this->id)) {
            $this->db->Select('*')
                     ->From($this->Pages)
                     ->WhereColName('uid')
                     ->WhereColValue($this->id)
                     ->Andd(str_replace("%%%n%%%", "", $this->and_lang) . " AND ((starttime = 0 AND endtime = 0 AND hidden != 1) OR (starttime != 0 AND starttime <= " . time() . " AND endtime = 0 AND hidden != 1) OR (starttime != 0 AND starttime <= " . time() . " AND endtime != 0 AND endtime >= " . time() . " AND hidden != 1) OR (starttime = 0 AND endtime != 0 AND endtime >= " . time() . " AND hidden != 1) OR (starttime != 0 AND starttime >= " . time() . " AND endtime = 0 AND hidden = 1) OR (starttime != 0 AND starttime >= " . time() . " AND endtime != 0 AND endtime <= " . time() . " AND hidden = 1) OR (starttime = 0 AND endtime != 0 AND endtime <= " . time() . " AND hidden = 1) OR (starttime != 0 AND starttime < " . time() . " AND endtime != 0 AND endtime <= " . time() . " AND hidden = 1) OR (starttime != 0 AND starttime >= " . time() . " AND endtime != 0 AND endtime > " . time() . " AND hidden = 1))")
                     ->Parameter($this->and_lang != '' ? array('lang' => $GLOBALS['LANGUAGE']) : array())
                     ->doSelect();
            $row = $this->db->SelectPuffer;
            $num = $this->db->Rows;
        } elseif (!empty($this->page_Hash) && $this->page_Hash != "") {
            $this->db->Select('p.*, p2.pagename as content_from_pid_name')
                     ->From($this->Pages . " as p")
                     ->WhereColName("p.page_Hash")
                     ->WhereColValue($this->page_Hash)
                     ->Joins("LEFT JOIN fe_" . $this->Pages . " as p2 ON p.content_from_pid = p2.uid")
                     ->Andd(str_replace("%%%n%%%", "p.", $this->and_lang) . " AND ((p.starttime = 0 AND p.endtime = 0 AND p.hidden != 1) OR (p.starttime != 0 AND p.starttime <= " . time() . " AND p.endtime = 0 AND p.hidden != 1) OR (p.starttime != 0 AND p.starttime <= " . time() . " AND p.endtime != 0 AND p.endtime >= " . time() . " AND p.hidden != 1) OR (p.starttime = 0 AND p.endtime != 0 AND p.endtime >= " . time() . " AND p.hidden != 1) OR (p.starttime != 0 AND p.starttime >= " . time() . " AND p.endtime = 0 AND p.hidden = 1) OR (p.starttime != 0 AND p.starttime >= " . time() . " AND p.endtime != 0 AND p.endtime <= " . time() . " AND p.hidden = 1) OR (p.starttime = 0 AND p.endtime != 0 AND p.endtime <= " . time() . " AND p.hidden = 1) OR (p.starttime != 0 AND p.starttime < " . time() . " AND p.endtime != 0 AND p.endtime <= " . time() . " AND p.hidden = 1) OR (p.starttime != 0 AND p.starttime >= " . time() . " AND p.endtime != 0 AND p.endtime > " . time() . " AND p.hidden = 1))")
                     ->Parameter($this->and_lang != '' ? array('lang' => $GLOBALS['LANGUAGE']) : array())
                     ->doSelect();
            
            $row = $this->db->SelectPuffer;
            $num = $this->db->Rows;
        } else {
            # Startseite
            $this->db->Select('*')
                     ->From($this->Pages)
                     ->WhereColName('spid')
                     ->WhereColValue($this->spid)
                     ->Andd(str_replace("%%%n%%%", "", $this->and_lang) . " AND ((starttime = 0 AND endtime = 0 AND hidden != 1) OR (starttime != 0 AND starttime <= " . time() . " AND endtime = 0 AND hidden != 1) OR (starttime != 0 AND starttime <= " . time() . " AND endtime != 0 AND endtime >= " . time() . " AND hidden != 1) OR (starttime = 0 AND endtime != 0 AND endtime >= " . time() . " AND hidden != 1) OR (starttime != 0 AND starttime >= " . time() . " AND endtime = 0 AND hidden = 1) OR (starttime != 0 AND starttime >= " . time() . " AND endtime != 0 AND endtime <= " . time() . " AND hidden = 1) OR (starttime = 0 AND endtime != 0 AND endtime <= " . time() . " AND hidden = 1) OR (starttime != 0 AND starttime < " . time() . " AND endtime != 0 AND endtime <= " . time() . " AND hidden = 1) OR (starttime != 0 AND starttime >= " . time() . " AND endtime != 0 AND endtime > " . time() . " AND hidden = 1))")
                     ->Parameter($this->and_lang != '' ? array('lang' => $GLOBALS['LANGUAGE']) : array())
                     ->doSelect();
            $row = $this->db->SelectPuffer;
            $num = $this->db->Rows;
        }

        
        if ($num) {

            # Template Pfad
            $this->TemplatePage = TEMPLATE_PATH . $row['sitetpl'];

            # Browser Titel
            if (!empty($row['BrowserTitle'])) {
                $BrowserTitle = $row['BrowserTitle'];
            } else {
                $BrowserTitle = BROWSER_NAME . " - " . $row['nav_title'];
            }

            $pageSettings = unserialize($row['pageSettings']);
            $this->PageCache = $pageSettings['pageCache'];
            $this->PageExpireCache = $pageSettings['expireCache'];
            $this->Index = $pageSettings['Index'];
            $this->JavaScript = stripslashes($row['Javascript']);
            $this->Stylesheet = stripslashes($row['Stylesheet']);
            $this->PageSettings = $pageSettings;

            $GLOBALS['PageConfig'] = array( 'UID' => $row['uid'],
                                            'PID' => $row['pid'],
                                            'PAGE_TITLE' => $row['nav_title'],
                                            'AUTH_USER_ID' => $row['perms_userid'],
                                            'AUTH_GROUP_ID' => $row['perms_groupid'],
                                            'PAGENAME' => $row['pagename'],
                                            'PAGE_URL' => $row['url'],
                                            'PAGE_URL_TYP' => $row['urltype'],
                                            'STARTTIME' => $row['starttime'],
                                            'ENDTIME' => $row['endtime'],
                                            'PAGE_URL_TARGET' => $row['target'],
                                            'NAVI_TITLE' => $row['nav_title'],
                                            'CONTENT_FROM_PID' => $row['content_from_pid'],
                                            'CONTENT_FROM_PID_NAME' => $row['content_from_pid_name'],
                                            'BUTTON' => $row['button'],
                                            'BUTTON_PATH' => $row['button_path'],
                                            'BUTTON_ACT_PATH' => $row['button_act_path'],
                                            'PAGE_TEMPLATE' => $row['sitetpl'],
                                            'MODULE' => $row['module'],
                                            'TITLE' => $BrowserTitle,
                                            'KEYWORDS' => $row['keywords'],
                                            'ROOT_FOLDER' => $this->RootFolder,
                                            'DOC_TYPE' => $row['docType'],
                                            'DESCRIPTION' => $row['description'],
                                            'PAGE_HASH' => $row['page_Hash'],
                                            'THIS_CSS' => $row['this_css'],
                                            'RGT' => $row['rgt'],
                                            'LFT' => $row['lft'],
                                            'PONDON_ID' => $row['rid']);
            $this->PageConfig = $GLOBALS['PageConfig'];

            # Pfadinfos werden geholt
            $this->getPathInformation($GLOBALS['PageConfig']['UID']);
            
            if ($GLOBALS['PageConfig']['CONTENT_FROM_PID'] != 0) {
                $this->ContentID = $GLOBALS['PageConfig']['CONTENT_FROM_PID'];
            } else {
                $this->ContentID = $GLOBALS['PageConfig']['UID'];
            }

            # Weiterleitung
            if (!empty($GLOBALS['PageConfig']['PAGE_URL']) && $GLOBALS['PageConfig']['PAGE_URL_TYP'] == 7) {
                $redirect = preg_match("#dotlink#ism", $GLOBALS['PageConfig']['PAGE_URL'], $hit) ? HTTPO . \lib\Functions::dotLink($GLOBALS['PageConfig']['PAGE_URL']) : $GLOBALS['PageConfig']['PAGE_URL'];
                
                header("HTTP/1.1 301 Moved Permanently");
                header("Location: " . $redirect);
                exit;
            }

            # Page Cache
            if (preg_match('#MSIE#', $_SERVER['HTTP_USER_AGENT'], $agent)) {
                $browser_type = $GLOBALS['BROWSER_TYPES']['MSIE'];
            } else {
                $browser_type = $GLOBALS['BROWSER_TYPES']['MOZ'];
            }
            
            $this->db->Select('*')
                     ->From('pages_cache')
                     ->WhereColName('uid')
                     ->WhereColValue($GLOBALS['PageConfig']['UID'])
                     ->Andd("AND lang = :lang AND browser = :browser ")
                     ->Parameter(array('lang' => $GLOBALS['LANGUAGE'], 'browser' => $browser_type))
                     ->doSelect();
            $row = $this->db->SelectPuffer;
            if (!$this->db->Rows && $this->PageCache != 1) {
                return true;
            } else {
                $this->CachedContent = stripslashes($row['HTML']);
                if ($row['expires'] > time()) {
                    $this->PageCacheUpdate = false;
                    return false;
                } else {
                    $this->PageCacheUpdate = true;
                    return true;
                }
            }
        } else {
            $this->TemplatePage = TEMPLATE_PATH . "error.tpl";
            $GLOBALS['PageConfig']['TITLE'] = "<title>" . BROWSER_NAME . " - Error!</title>";
            $this->Error = 1;
        }
    }

    /*
     * Hilfsfunktion für Pfadinformationen
     *
     */

    public function getPathInformation($uid) {
        $this->db->TablePreName = 'fe_';
        $this->db->Select("p.*")
                 ->From($this->Pages . " n, fe_" . $this->Pages . " p")
                 ->noWhere(1)
                 ->Andd("WHERE n.lft BETWEEN p.lft AND p.rgt AND n.uid = :uid " . str_replace("%%%n%%%", "p.", $this->and_lang))
                 ->Order('ORDER BY p.lft')
                 ->Parameter($this->and_lang != '' ? array('lang' => $GLOBALS['LANGUAGE'], 'uid' => $uid) : array('uid' => $uid))
                 ->SelectMethod(1)
                 ->doSelect();
        
        $num = $this->db->Rows;
        if($num) {
            foreach ($this->db->SelectPuffer as $key => $row) {
                $ouri = $uri;
                $this->PageTreePointsHashes[$i]         = $row['page_Hash'];
                $this->PageTreePointsLevel[$i]          = $row['uid'];
                $this->PageTreePoints[$i]['uri']        = $uri . "/" . $row['pagename'] . ".html";
                $this->PageTreePoints[$i]['hash']       = $row['page_Hash'];
                $this->PageTreePoints[$i]['uid']        = $row['uid'];
                $this->PageTreePoints[$i]['pagename']   = $row['pagename'];
                $uri = $ouri . "/" . $row['pagename'];
                
            }
        }
    }

    /*
     * HTML-Header
     * 		
     */

    public function HTMLHeader() {
        
        $this->db->Select('*')
                 ->From('html_header')
                 ->WhereColName('client_id')
                 ->WhereColValue($GLOBALS['CLIENT_ID'])
                 ->Andd("AND lang = :lang ")
                 ->Parameter(array('lang' => $GLOBALS['LANGUAGE']))
                 ->doSelect();
        $num = $this->db->Rows;
        if ($num) {
            $row = $this->db->SelectPuffer;
            $oldTemp = Functions::readFiles("/" . $this->TemplatePage);
            preg_match('#<head[^>]*>(.+)</head>#is', $oldTemp, $headers);
            if (preg_match('#MSIE#', $_SERVER['HTTP_USER_AGENT'], $agent)) {
                $css_file = $row['stylesheet'];
            } else {
                $css_file = $row['stylesheet_ff'];
            }
            $css = Functions::readFiles("/fileadmin/css/".$GLOBALS['CLIENT_TEMPLATE_FOLDER']."". $css_file);
            if (!empty($this->BrowserTitle)) {
                $title = $this->BrowserTitle . " - ";
            }
            $header .= "<title>" . $title . $GLOBALS['PageConfig']['TITLE'] . "</title>\n";
            if (!empty($GLOBALS['PageConfig']['KEYWORDS'])) {
                preg_match('#<meta\s*name="keywords"\s*content="(.*?)["](.*?)>#', stripslashes($row['metatags']), $keywords);
                $metatags = str_replace($keywords[1], $GLOBALS['PageConfig']['KEYWORDS'], stripslashes($row['metatags']));
            } else {
                $metatags = stripslashes($row['metatags']);
            }
            if (!empty($this->Description)) {
                preg_match('#<meta\s*name="description"\s*content="(.*?)["](.*?)>#', $metatags, $description);
                preg_match('#<meta\s*name="DC.Description"\s*content="(.*?)["](.*?)>#', $metatags, $description_DC);
                $metatags = str_replace($description[1], $this->Description . " " . $GLOBALS['PageConfig']['DESCRIPTION'], $metatags);
                $metatags = str_replace($description_DC[1], $this->Description . " " . $GLOBALS['PageConfig']['DESCRIPTION'], $metatags);
            } elseif (!empty($GLOBALS['PageConfig']['DESCRIPTION'])) {
                preg_match('#<meta\s*name="description"\s*content="(.*?)["](.*?)>#', $metatags, $description);
                preg_match('#<meta\s*name="DC.Description"\s*content="(.*?)["](.*?)>#', $metatags, $description_DC);
                $metatags = str_replace($description[1], $GLOBALS['PageConfig']['DESCRIPTION'], $metatags);
                $metatags = str_replace($description_DC[1], $GLOBALS['PageConfig']['DESCRIPTION'], $metatags);
            }

            if (!empty($this->Index)) {
                switch ($this->Index) {
                    case "1":
                        preg_match_all('#<meta\s*name="robots"\s*content="(.*?)["](.*?)>#is', $metatags, $indexfollow);
                        $metatags = str_replace($indexfollow[0][0], "<meta name=\"robots\" content=\"index, follow\">", $metatags);

                        break;

                    case "2":
                        preg_match_all('#<meta\s*name="robots"\s*content="(.*?)["](.*?)>#is', $metatags, $indexfollow);
                        $metatags = str_replace($indexfollow[0][0], "<meta name=\"robots\" content=\"noindex, nofollow\">", $metatags);

                        break;
                }
            }

            if ($GLOBALS['PageConfig']['THIS_CSS'] != 1) {
                $header .= ARIBU_COPYRIGHTS . "\n" . $metatags . "\n<style type=\"text/css\"><!--\n" . stripslashes($css) . "\n--></style>\n" . stripslashes($row['jscripts']) . "\n" . $this->JavaScript . "\n" . $this->Stylesheet . "\n";
            } else {
                $header .= ARIBU_COPYRIGHTS . "\n" . $metatags . "\n" . $this->Stylesheet . "\n" . stripslashes($row['jscripts']) . "\n" . $this->JavaScript . "\n";
            }
            $this->HTMLOldHeader = $headers[1];
            $TemplateAssignArray = array($headers[1] => $header);
            $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
        }
    }

    /*
     * Content-Builder
     *
     */

    public function Content(array $ContentMarker, $c='', $flag=1, $and='') {
        
        $session_user_rights = isset($_SESSION['sess_user_rights']) ? $_SESSION['sess_user_rights'] : 0;
        

        foreach ($ContentMarker as $key => $CMarkerval) {
            
            if ($GLOBALS['PageConfig']['AUTH_GROUP_ID'] <= $session_user_rights) {
                if ($GLOBALS['PageConfig']['AUTH_USER_ID'] == $_SESSION['sess_user_id'] || $GLOBALS['PageConfig']['AUTH_USER_ID'] == 0) {

                    $order = $key == '0' ? "ORDER BY sorting" : "ORDER BY sorting";
                    $this->db->Select('*')
                             ->From('content')
                             ->WhereColName('pid')
                             ->WhereColValue($this->ContentID)
                             ->Andd(  "AND colP = :colp "
                                    . "AND lang = :lang "
                                    . "AND ("
                                    . "(starttime = 0 AND endtime = 0 AND hidden != '1') "
                                    . "OR (starttime != 0 AND starttime <= " . time() . " AND endtime = 0 AND hidden != '1') "
                                    . "OR (starttime != 0 AND starttime <= " . time() . " AND endtime != 0 AND endtime >= " . time() . " AND hidden != '1') "
                                    . "OR (starttime = 0 AND endtime != 0 AND endtime >= " . time() . " AND hidden != '1') "
                                    . "OR (starttime != 0 AND starttime >= " . time() . " AND endtime = 0 AND hidden = '1') "
                                    . "OR (starttime != 0 AND starttime >= " . time() . " AND endtime != 0 AND endtime <= " . time() . " AND hidden = '1') "
                                    . "OR (starttime = 0 AND endtime != 0 AND endtime <= " . time() . " AND hidden = '1') "
                                    . "OR (starttime != 0 AND starttime < " . time() . " AND endtime != 0 AND endtime <= " . time() . " AND hidden = '1') "
                                    . "OR (starttime != 0 AND starttime >= " . time() . " AND endtime != 0 AND endtime > " . time() . " AND hidden = '1')) ".$and)
                             ->Order($order)
                             ->SelectMethod(1)
                             ->Parameter(array('colp' => $key, 'lang' => $GLOBALS['LANGUAGE']))
                             ->doSelect();
                    $num = $this->db->Rows;
                    
                    if ($num > 0) {
                        $row = $this->db->SelectPuffer;
                        for ($i = 0; $i < $num; $i++) {

                            if ($row[$i]['perms_groupid'] <= $session_user_rights) {
                                $BlockData = '';
                                
                                # Dot-Content
                                switch ($row[$i]['CType']) {
                                    case "HTML":
                                        $BlockData = unserialize($row[$i]['script_method']);

                                        $h1 = isset($GLOBALS['CONTENT_MOD_TXTI_HEADLINES'][1]) ? $GLOBALS['CONTENT_MOD_TXTI_HEADLINES'][1] : 'h1';
                                        $h2 = isset($GLOBALS['CONTENT_MOD_TXTI_HEADLINES'][2]) ? $GLOBALS['CONTENT_MOD_TXTI_HEADLINES'][2] : 'h2';
                                        $headlines = "";
                                        $headlines = $row[$i]['headline'] != "" ? '<' . $h1 . '>' . $row[$i]['headline'] . '</' . $h1 . '>' : "";
                                        $headlines .= $row[$i]['subline'] != "" ? '<' . $h2 . '>' . $row[$i]['subline'] . '</' . $h2 . '>' : "";

                                        $block_style = "";
                                        $clear = '<br style="clear:both;" />';

                                        $block_style .= $BlockData['block_width'] != "" ? "width:" . $BlockData['block_width'] . "; " : "";
                                        $block_style .= $BlockData['block_height'] != "" ? "height:" . $BlockData['block_height'] . "; " : "";

                                        $border_width = $BlockData['border_width'] != "" ? $BlockData['border_width'] : '1';
                                        $border_color = $BlockData['border_color'] != "" ? '' . $BlockData['border_color'] . '' : '';
                                        $border_style = $BlockData['border_style'] != "" ? '' . $BlockData['border_style'] . ' ' : '';

                                        if (isset($BlockData['border'])) {
                                            switch ($BlockData['border']) {
                                                case "1": $block_style .= 'border-top:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                                case "2": $block_style .= 'border-right:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                                case "3": $block_style .= 'border-left:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                                case "4": $block_style .= 'border-bottom:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                                case "5": $block_style .= 'border-top:' . $border_width . 'px ' . $border_style . $border_color . '; border-bottom:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                                case "6": $block_style .= 'border-top:' . $border_width . 'px ' . $border_style . $border_color . '; border-right:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                                case "7": $block_style .= 'border-top:' . $border_width . 'px ' . $border_style . $border_color . '; border-left:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                                case "8": $block_style .= 'border-bottom:' . $border_width . 'px ' . $border_style . $border_color . '; border-right:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                                case "9": $block_style .= 'border-bottom:' . $border_width . 'px ' . $border_style . $border_color . '; border-left:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                                case "10": $block_style .= 'border-right:' . $border_width . 'px ' . $border_style . $border_color . '; border-left:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                                case "11": $block_style .= 'border:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                                    break;
                                            }
                                        }

                                        $block_style .= $BlockData['block_padding'] != "" ? "padding:" . $BlockData['block_padding'] . "; " : "";
                                        $block_style .= $BlockData['block_margin'] != "" ? "margin:" . $BlockData['block_margin'] . "; " : "";
                                        
                                        $block_id = $BlockData['block_id'] != '' ? 'id="'.$BlockData['block_id'].'" ' : '';
                                        
                                        if ($BlockData['block_position'] != "") {
                                            switch ($BlockData['block_position']) {
                                                case "1":
                                                    $block_style .= "float:left; ";
                                                    $block_start = '<div class="dor-html-block '.$BlockData['block_class'].'" style="' . $block_style . '" '.$block_id.'>';
                                                    $block_end = '<br style="clear:both;" /></div>';
                                                    break;

                                                case "2":
                                                    $block_style .= " text-align:center;";
                                                    $block_start = '<div class="dor-html-block '.$BlockData['block_class'].'" style="' . $block_style . '" '.$block_id.'><div style="margin:auto; display:inline-block;">';
                                                    $block_end = '<br style="clear:both;" /></div>' . $clear;
                                                    break;

                                                case "3":
                                                    $block_style .= "float:right;";
                                                    $block_start = '<div class="dor-html-block '.$BlockData['block_class'].'" style="' . $block_style . '" '.$block_id.'>';
                                                    $block_end = '<br style="clear:both;" /></div>';
                                                    break;
                                            }
                                        }


                                        $ContentTag = $block_start . $headlines . stripslashes(Functions::replaceLinks($row[$i]['bodytext'], "dotLink")) . $block_end;

                                        if (!isset($this->FID) || $this->FID == "") {
                                            $CTag .= $ContentTag;
                                        } else {
                                            if ($key != 0) {
                                                $CTag .= $ContentTag;
                                            }
                                        }


                                        break;

                                    case "PHP":
                                        
                                        $PHP_Path = DOC_ROOT . "/scripts/" . $row[$i]['script_path'];
                                        $this->PHPTag = 1;
                                        $script_marker = str_replace("###", "", $GLOBALS['CONTENT_MARKER'][$key]);
                                        $script_marker = "###" . $script_marker . "_" . $i . $c . "###";
                                        $GLOBALS['PHPMethod'][$script_marker] = $row[$i]['script_method'];
                                        $GLOBALS['PHPBlock'][$script_marker] = $row[$i]['bodytext'];
                                        $PHPMarkerPath = array($script_marker => $PHP_Path);
                                        $this->PHPMarkerPath = array_merge($this->PHPMarkerPath, $PHPMarkerPath);
                                        $CTag .= $script_marker;
                                        break;

                                    case "MENU":
                                        $GLOBALS['MenuConfig'][1000] = unserialize($row[$i]['script_method']);
                                        
                                        if ((!isset($GLOBALS['MenuConfig'][1000]['show_on_detail']) && $this->FID == "") || ($GLOBALS['MenuConfig'][1000]['show_on_detail'] != 1 || $this->FID == "")) {
                                            $ContentMenu = new fe_menu();
                                            $ContentMenu->AdminShow = $this->AdminShowHidden;
                                        
                                            $ContentMenuObj = $ContentMenu->$GLOBALS['MenuConfig'][1000]['Type'](1000);
                                            $CTag .= $ContentMenuObj[$GLOBALS['MenuConfig'][1000]['Marker']];
                                            unset($GLOBALS['MenuConfig']);
                                        } else {
                                            $CTag .= '';
                                        }
                                        break;


                                    default:                                        
                                        $ContentTag = stripslashes($row[$i]['bodytext']);
                                        $BlockData = unserialize(stripslashes($row[$i]['script_method']));
                                        if ((!isset($BlockData['show_on_detail']) && $this->FID == "") || ($BlockData['show_on_detail'] != 1 || $this->FID == "")) {
                                            if ($BlockData['classname'] != "") {
                                                if (isset($this->Counter[$BlockData['classname']])) {
                                                    $this->Counter[$BlockData['classname']] ++;
                                                } else {
                                                    $this->Counter[$BlockData['classname']] = 1;
                                                }
                                            }

                                            $BlockData['CID'] = $row[$i]['uid'];
                                            $BlockData['COLP'] = $key;
                                            $BlockData['Content'] = $ContentTag;
                                            $BlockData['Headline'] = stripslashes($row[$i]['headline']);
                                            $BlockData['Subline'] = stripslashes($row[$i]['subline']);
                                            $BlockData['Sorting'] = $row[$i]['sorting'];
                                            
                                            if(file_exists(DOC_ROOT . $BlockData['script_path'])) {
                                               include(DOC_ROOT . $BlockData['script_path']); 
                                            } else {
                                               $pluginCObj = 'Include-Error: Cannot include this script ' . $BlockData['script_path'];
                                            }
                                            
                                            if ($BlockData['classname'] != "") {
                                                $ContModul = new $BlockData['classname'];
                                                $ContModul->GlobalVars = $BlockData;
                                                $CTag .= $ContModul->initClass();
                                            } else {
                                                $CTag .= $pluginCObj;
                                            }
                                        } else {
                                            $CTag .= '';
                                        }
                                        
                                }
                            } else {
                                # User hat keinen Zutritt
                                $template_not_logged_in = $this->PageSettings['Template_Not_Logged_In'] != '' ? '/_templates/' . $this->PageSettings['Template_Not_Logged_In'] : '/_templates/'.$GLOBALS['CLIENT_TEMPLATE_FOLDER'].'error/norights.html';
                                $CTag .= Functions::replaceLinks(Functions::readFiles($template_not_logged_in));
                            }
                            
                            $this->CTag .= $CTag;
                            $CTag = '';
                        }
                        
                        
                        $this->ColPosition = $key;
                        if($flag == 1) {
                            $TemplateAssignArray = array($GLOBALS['CONTENT_MARKER'][$key] => $this->CTag);
                            $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                            unset($CTag);
                            unset($TemplateAssignArray);
                            unset($PHPMarkerPath);
                            $this->CTag = '';
                        }
                    } else {
                        if (!empty($GLOBALS['PageConfig']['MODULE'])) {
                            if ($CMarkerval == "###CONTENT###") {
                                switch ($GLOBALS['PageConfig']['MODULE']) {

                                    default:
                                        $db = $this->db;
                                        # Installierte Module im Plugin-Ordner werden geladen
                                        
                                        if (file_exists(DOC_ROOT . '/plugins/' . strtolower($GLOBALS['PageConfig']['MODULE']) . '/index.php')) {
                                            include(DOC_ROOT . '/plugins/' . strtolower($GLOBALS['PageConfig']['MODULE']) . '/index.php');
                                            $TemplateAssignArray = array("###CONTENT###" => $pluginCObj);
                                            $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                                        } else {
                                            $TemplateAssignArray = array("###CONTENT###" => "Konnte das Plugin nicht einbinden. Das Plugin ist nicht im Plugin-Ordner vorhanden.<br>Pfad: ".DOC_ROOT . '/plugins/' . strtolower($GLOBALS['PageConfig']['MODULE']) . '/index.php');
                                            $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                                        }
                                }
                            } else {
                                # Falls kein Content vorhanden
                                $TemplateAssignArray = array($CMarkerval => '');
                                $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                            }
                        } else {
                            # Falls kein Content vorhanden
                            $TemplateAssignArray = array($CMarkerval => '');
                            $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                        }
                    }
                } else {
                    # User hat keinen Zutritt
                    $template_not_logged_in = $this->PageSettings['Template_Not_Logged_In'] != '' ? '/_templates/' . $this->PageSettings['Template_Not_Logged_In'] : '/_templates/'.$GLOBALS['CLIENT_TEMPLATE_FOLDER'].'error/norights.html';
                    $contemp = $CMarkerval == "###CONTENT###" ? Functions::replaceLinks($template_not_logged_in) : '';
                    if($contemp != '') {
                        $ContTemp = $this->TT->getFilledTemplate($contemp);
                    } else {
                        $ContTemp = '';
                    }                
                    $TemplateAssignArray = array($CMarkerval => $ContTemp);
                    $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                }
            } else {
                # User hat keinen Zutritt
                if ($CMarkerval == "###CONTENT###") {
                    switch ($GLOBALS['PageConfig']['MODULE']) {

                        default:
                            # Installierte Module werden geladen
                            if (file_exists(DOC_ROOT . '/plugins/' . strtolower($GLOBALS['PageConfig']['MODULE']) . '/index.php')) {
                                if (strtolower($GLOBALS['PageConfig']['MODULE']) != 'faq') {
                                    include(DOC_ROOT . '/plugins/' . strtolower($GLOBALS['PageConfig']['MODULE']) . '/index.php');
                                }
                            }
                    }
                }
                if ($this->jumpURL == "") {
                    $template_not_logged_in = $this->PageSettings['Template_Not_Logged_In'] != '' ? '/_templates/' . $this->PageSettings['Template_Not_Logged_In'] : '/_templates/'.$GLOBALS['CLIENT_TEMPLATE_FOLDER'].'error/norights.html';
                    $contemp = $CMarkerval == "###CONTENT###" ? Functions::replaceLinks($template_not_logged_in) : '';
                    
                    if($contemp != '') {
                        $ContTemp = $this->TT->getFilledTemplate($contemp);
                    } else {
                        $ContTemp = '';
                    }
                    $TemplateAssignArray = array($CMarkerval => $ContTemp);
                    $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                } else {
                    header("Location: " . $this->jumpURL);
                    exit;
                }
            }
        } # Ende foreach
    } # Ende function



    /*
     * ElementBuilder
     *
     */

    public function Elements() {
        $db = $this->db;
        $DFE = $this;
        
        $this->db->Select("*")
                 ->From("elements")
                 ->WhereColName("client_id")
                 ->WhereColValue($GLOBALS['CLIENT_ID'])
                 ->SelectMethod(1)
                 ->doSelect();
        $num = $this->db->Rows;

        if ($num) {
            foreach ($this->db->SelectPuffer as $key => $element) {
                $this->Element_Attributes = unserialize($element['element_attributes']);
                $pages_show_element = explode("-", $this->Element_Attributes['pages_show_element']);
                $this_page_show = 0;
                if (is_array($this->PageTreePointsLevel)) {
                    foreach ($this->PageTreePointsLevel as $key => $val) {
                        if (in_array($val, $pages_show_element)) {
                            $this_page_show = 1;
                        }
                    }
                }

                if ($this_page_show == 1 || empty($this->Element_Attributes['pages_show_element']) || $this->Element_Attributes['pages_show_element'] == 0) {
                    switch ($element['element_content']) {
                        case 0:
                            if ($element['hidden'] != 1) {
                                $sc_method = $element['class_methods'];
                                $sc_marker = $element['marker'];
                                
                                include(DOC_ROOT . "/elements/" . $element['element_path']);
                                # Assign Module
                                $TemplateAssignArray = array($element['marker'] => $GLOBALS['ELEMENT_TAG'][$element['marker']]);
                                $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                                unset($TemplateAssignArray);
                                unset($GLOBALS['ELEMENT_TAG']);
                            } else {
                                $TemplateAssignArray = array("" . $element['marker'] . "" => '');
                                $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                                unset($TemplateAssignArray);
                                unset($GLOBALS['ELEMENT_TAG']);
                            }
                            break;

                        case 1:
                            if ($element['hidden'] != 1) {
                                $element_content = $this->ContentElement($element['pid'], $element['marker']);
                                $TemplateAssignArray = array($element['marker'] => $element_content);
                                $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                                unset($TemplateAssignArray);
                            } else {
                                $element_content = "";
                                $TemplateAssignArray = array($element['marker'] => $element_content);
                                $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                                unset($TemplateAssignArray);
                            }
                            break;

                        case 2:
                            if ($element['hidden'] != 1) {
                                $sc_method = $element['class_methods'];
                                $sc_marker = $element['marker'];
                                
                                include(DOC_ROOT . "/elements/" . $element['element_path']);
                                foreach ($this->TemplateAssignArray as $marker_key => $content_val) {
                                    if (preg_match('/' . $element['marker'] . '/', $content_val, $treffer)) {
                                        $dot = $this->TemplateAssignArray[$marker_key];
                                        $dot = str_replace($element['marker'], $GLOBALS['ELEMENT_TAG'][$element['marker']], $dot);
                                        $this->TemplateAssignArray[$marker_key] = $dot;
                                    }
                                }
                            } else {
                                $element_content = "";
                            }
                            break;

                        case 3:
                            if ($element['hidden'] != 1) {
                                $element_content = $this->ContentElement($element['pid']);
                            } else {
                                $element_content = "";
                            }
                            $element_content_buffer[] = $element_content;
                            $element_buffer[] = $element['marker'];
                            foreach ($this->TemplateAssignArray as $marker_key => $content_val) {
                                if (preg_match('/' . $element['marker'] . '/', $content_val, $treffer)) {
                                    $dot = $this->TemplateAssignArray[$marker_key];
                                    $dot = str_replace($element['marker'], $element_content, $dot);
                                    $this->TemplateAssignArray[$marker_key] = $dot;
                                }
                            }
                            break;
                    }
                } else {
                    switch ($element['module_content']) {
                        case 0:
                            $TemplateAssignArray = array("" . $element['marker'] . "" => '');
                            $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                            unset($TemplateAssignArray);
                            unset($GLOBALS['ELEMENT_TAG']);
                            break;

                        case 1:
                            $element_content = "";
                            $TemplateAssignArray = array($element['marker'] => $element_content);
                            $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                            unset($TemplateAssignArray);
                            break;

                        case 2:
                            $element_content = "";
                            break;

                        case 3:
                            $element_content = "";
                            $element_content_buffer[] = $element_content;
                            $element_buffer[] = $element['marker'];
                            foreach ($this->TemplateAssignArray as $marker_key => $content_val) {
                                if (preg_match('/' . $element['marker'] . '/', $content_val, $treffer)) {
                                    $dot = $this->TemplateAssignArray[$marker_key];
                                    $dot = str_replace($element['marker'], $element_content, $dot);
                                    $this->TemplateAssignArray[$marker_key] = $dot;
                                }
                            }
                            break;
                    }
                }
            }
            $this->ElementContent = $element_content_buffer;
            $this->ElementMarker = $element_buffer;
        }
    }

    /*
     * Content Element
     *
     */

    public function ContentElement($cid) {

        if (SYS_LANGUAGE != $GLOBALS['LANGUAGE']) {
            $this->db->Select('uid')
                     ->From('pages_language')
                     ->WhereColName('rid')
                     ->WhereColValue($cid)
                     ->Andd("AND lang = :lang ")
                     ->Parameter(array('lang' => $GLOBALS['LANGUAGE']))
                     ->doSelect();
            $cid = $this->db->SelectPuffer['uid'];
        }

        $this->Element = 1;

        $order = $this->Element_Attributes['rotate_content'] == 1 ? "ORDER BY RAND()" : "ORDER BY sorting";
        $limit = !empty($this->Element_Attributes['content_limit']) ? "LIMIT " . $this->Element_Attributes['content_limit'] : "";

        $this->db->Select('*')
                 ->From('content')
                 ->WhereColName('pid')
                 ->WhereColValue($cid)
                 ->Andd("AND colP = 0 "
                    . "AND lang = '" . $GLOBALS['LANGUAGE'] . "' "
                    . "AND ((starttime = 0 AND endtime = 0 AND hidden != 1) "
                    . "OR (starttime != 0 AND starttime <= " . time() . " AND endtime = 0 AND hidden != 1) "
                    . "OR (starttime != 0 AND starttime <= " . time() . " AND endtime != 0 AND endtime >= " . time() . " AND hidden != 1) "
                    . "OR (starttime = 0 AND endtime != 0 AND endtime >= " . time() . " AND hidden != 1) "
                    . "OR (starttime != 0 AND starttime >= " . time() . " AND endtime = 0 AND hidden = 1) "
                    . "OR (starttime != 0 AND starttime >= " . time() . " AND endtime != 0 AND endtime <= " . time() . " AND hidden = 1) "
                    . "OR (starttime = 0 AND endtime != 0 AND endtime <= " . time() . " AND hidden = 1) "
                    . "OR (starttime != 0 AND starttime < " . time() . " AND endtime != 0 AND endtime <= " . time() . " AND hidden = 1) "
                    . "OR (starttime != 0 AND starttime >= " . time() . " AND endtime != 0 AND endtime > " . time() . " AND hidden = 1))")
                 ->Order($order)
                 ->Limit($limit)
                 ->SelectMethod(1)
                 ->doSelect();
        $row = $this->db->SelectPuffer;
        $num = $this->db->Rows;
        if ($num > 0) {
            for ($i = 0; $i < $num; $i++) {
                # Dot-Content
                switch ($row[$i]['CType']) {

                    case "HTML":
                        $BlockData = unserialize($row[$i]['script_method']);

                        $h1 = isset($GLOBALS['CONTENT_MOD_TXTI_HEADLINES'][1]) ? $GLOBALS['CONTENT_MOD_TXTI_HEADLINES'][1] : 'h1';
                        $h2 = isset($GLOBALS['CONTENT_MOD_TXTI_HEADLINES'][2]) ? $GLOBALS['CONTENT_MOD_TXTI_HEADLINES'][2] : 'h2';
                        $headlines = "";
                        $headlines = $row[$i]['headline'] != "" ? '<' . $h1 . '>' . $row[$i]['headline'] . '</' . $h1 . '>' : "";
                        $headlines .= $row[$i]['subline'] != "" ? '<' . $h2 . '>' . $row[$i]['subline'] . '</' . $h2 . '>' : "";

                        $block_style = "";
                        $clear = '<br style="clear:both;" />';

                        $block_style .= $BlockData['block_width'] != "" ? "width:" . $BlockData['block_width'] . "; " : "";
                        $block_style .= $BlockData['block_height'] != "" ? "height:" . $BlockData['block_height'] . "; " : "";

                        $border_width = $BlockData['border_width'] != "" ? $BlockData['border_width'] : '1';
                        $border_color = $BlockData['border_color'] != "" ? '' . $BlockData['border_color'] . '' : '';
                        $border_style = $BlockData['border_style'] != "" ? '' . $BlockData['border_style'] . ' ' : '';

                        if (isset($BlockData['border'])) {
                            switch ($BlockData['border']) {
                                case "1": $block_style .= 'border-top:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                                case "2": $block_style .= 'border-right:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                                case "3": $block_style .= 'border-left:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                                case "4": $block_style .= 'border-bottom:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                                case "5": $block_style .= 'border-top:' . $border_width . 'px ' . $border_style . $border_color . '; border-bottom:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                                case "6": $block_style .= 'border-top:' . $border_width . 'px ' . $border_style . $border_color . '; border-right:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                                case "7": $block_style .= 'border-top:' . $border_width . 'px ' . $border_style . $border_color . '; border-left:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                                case "8": $block_style .= 'border-bottom:' . $border_width . 'px ' . $border_style . $border_color . '; border-right:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                                case "9": $block_style .= 'border-bottom:' . $border_width . 'px ' . $border_style . $border_color . '; border-left:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                                case "10": $block_style .= 'border-right:' . $border_width . 'px ' . $border_style . $border_color . '; border-left:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                                case "11": $block_style .= 'border:' . $border_width . 'px ' . $border_style . $border_color . '; ';
                                    break;
                            }
                        }

                        $block_style .= $BlockData['block_padding'] != "" ? "padding:" . $BlockData['block_padding'] . "; " : "";
                        $block_style .= $BlockData['block_margin'] != "" ? "margin:" . $BlockData['block_margin'] . "; " : "";

                        if ($BlockData['block_position'] != "") {
                            switch ($BlockData['block_position']) {
                                case "1":
                                    $block_style .= "float:left; ";
                                    $block_start = '<div class="dor-html-block" style="' . $block_style . '">';
                                    $block_end = '<br style="clear:both;" /></div>';
                                    break;

                                case "2":
                                    $block_style .= " text-align:center;";
                                    $block_start = '<div class="dor-html-block" style="' . $block_style . '"><div style="margin:auto; display:inline-block;">';
                                    $block_end = '<br style="clear:both;" /></div>' . $clear;
                                    break;

                                case "3":
                                    $block_style .= "float:right;";
                                    $block_start = '<div class="dor-html-block" style="' . $block_style . '">';
                                    $block_end = '<br style="clear:both;" /></div>';
                                    break;
                            }
                        }


                        $ContentTag = $block_start . $headlines . stripslashes(Functions::replaceLinks($row[$i]['bodytext'], "dotLink")) . $block_end;
                        $ContentElementTag .= $ContentTag;

                        break;

                    case "MENU":
                        $ContentMenu = new fe_menu();
                        $ContentMenu->AdminShow = $this->AdminShowHidden;
                        $GLOBALS['MenuConfig'][10000] = unserialize($row[$i]['script_method']);
                        $ContentMenuObj = $ContentMenu->$GLOBALS['MenuConfig'][10000]['Type'](10000);
                        $ContentElementTag .= $ContentMenuObj[$GLOBALS['MenuConfig'][10000]['Marker']];
                        unset($GLOBALS['MenuConfig']);
                        break;

                    default:
                        $DFE = $this;
                        $ContentTag = stripslashes($row[$i]['bodytext']);
                        $BlockData = unserialize($row[$i]['script_method']);

                        if ($BlockData['classname'] != "") {
                            if (isset($this->Counter[$BlockData['classname']])) {
                                $this->Counter[$BlockData['classname']] ++;
                            } else {
                                $this->Counter[$BlockData['classname']] = 1;
                            }
                        }
                       
                        $BlockData['CID'] = $row[$i]['uid'];
                        $BlockData['COLP'] = $key;
                        $BlockData['Content'] = $ContentTag;
                        $BlockData['Headline'] = stripslashes($row[$i]['headline']);
                        $BlockData['Subline'] = stripslashes($row[$i]['subline']);
                        $BlockData['Sorting'] = $row[$i]['sorting'];
                        
                        $script_path = $BlockData['script_path'] != '' ? $BlockData['script_path'] : $row[$i]['script_path'];
                        
                        if(file_exists(DOC_ROOT . $script_path)) {
                            
                            include(DOC_ROOT . $script_path);
                        } elseif(file_exists(DOC_ROOT . '/scripts/' . $script_path)) {
                            
                            include(DOC_ROOT . '/scripts/' . $script_path);
                        }

                        if ($BlockData['classname'] != "") {
                            
                            $ContentElementObj = new $BlockData['classname'];
                            $ContentElementObj->GlobalVars = $BlockData;
                            $ContentElementTag .= $ContentElementObj->initClass();
                        } else {
                            
                            $ContentElementTag .= $pluginCObj;
                        }
                }
            }
            return $ContentElementTag;
        }
    }

    /*
     * Menu
     *
     */

    public function Menu() {

        # Menu Konfiguration aus der Datenbank lesen
        $this->db->Select('*')
                 ->From('menu_config')
                 ->WhereColName('client_id')
                 ->WhereColValue($GLOBALS['CLIENT_ID'])
                 ->Andd("AND menu_lang = :menulang ")
                 ->Parameter(array('menulang' => $GLOBALS['LANGUAGE']))
                 ->SelectMethod(1)
                 ->doSelect();
        $num = $this->db->Rows;
        if ($num) {
            $row = $this->db->SelectPuffer;
            for ($i = 0; $i < $this->db->Rows; $i++) {
                if (!empty($row[$i]['menu_key'])) {
                    $GLOBALS['MenuConfig'][$row[$i]['menu_title']][$row[$i]['menu_key']] = $row[$i]['menu_value'];
                } else {
                    $GLOBALS['MenuConfig'][$row[$i]['menu_title']] = $row[$i]['menu_value'];
                }
            }

            $Menu = new fe_menu($this);
            $Menu->AdminShow = $this->AdminShowHidden;
            $countmenu = count($GLOBALS['MenuConfig']);
            foreach ($GLOBALS['MenuConfig'] as $mkey => $mval) {
                if ($GLOBALS['MenuConfig'][$mkey]['Status'] == 1) {
                    $CMenuObj[$mkey] = $Menu->{$GLOBALS['MenuConfig'][$mkey]['Type']}($mkey);
                } else {
                    $CMenuObj[$mkey] = array("" . $GLOBALS['MenuConfig'][$mkey]['Marker'] . "" => "");
                }
            }
            if (is_array($CMenuObj)) {
                foreach ($CMenuObj as $key => $val) {
                    if (is_array($val)) {
                        foreach ($val as $kkey => $vval) {
                            if ($GLOBALS['MenuConfig'][$key]['MenuInContent'] == 1 && isset($GLOBALS['MenuConfig'][$key]['MenuInContent'])) {
                                $dot = str_replace($GLOBALS['MenuConfig'][$key]['Marker'], $vval, $this->TemplateAssignArray[$GLOBALS['MenuConfig'][$key]['MenuContentMarker']]);
                                $TemplateAssignArray = array($GLOBALS['MenuConfig'][$key]['MenuContentMarker'] => $dot);
                            } else {
                                $TemplateAssignArray = array($kkey => $vval);
                            }
                            $this->TemplateAssignArray = array_merge($this->TemplateAssignArray, $TemplateAssignArray);
                        }
                    }
                }
            }
        }
    }
    
    public function __toString() {
        $class_vars = get_class_vars(get_class($this));
        
        $string = '';
        $string .= '<script type="text/javascript">
	$(document).ready(function() {
	var winheight = $("html").innerHeight();
 	$("div#debug-div").css("height", winheight+"px");
	});
	</script>';
        $string .= '<pre style="display:block; position:absolute; z-index:1000; border:6px solid #efefef; background:#fcfcfc; padding:22px; top:0px; left:0px;">';
        $string .= '<div style="display:block; width:600px; height:500px; overflow:auto; background:#fcfcfc; text-align:left; font-size:11px; font-family:helvetica, verdana, sans-serif; line-height:14px; color:#666666;" id="debug-div">';
           
        $string .= "Price: \n";
        foreach($class_vars as $key => $val) {
            $values = !is_object($this->{$key}) ? print_r($this->{$key}, true) : 'Object';
            $string .= $key.': '.$values."\n";
        }
        $string .= '</div></pre>';
        return $string;
    }
}