<?php

namespace lib;

/*
 * aribus systems - Version 7.0.0
 * 
 * Copyright (C) 2018  Sascha Momcilovic
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 3 of the License, or (at your option) 
 * any later version. This program is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, see <http://www.gnu.org/licenses/>. 
 * 

 *
 * @author: Sascha Momcilovic
 * @e-Mail: info@aribu.systems
 * @link:   https://aribu.systems
 * @date:   26.10.2005
 * @update: 02.04.2015
 */

class Functions {
    /*
     * Simple URI explode and returns the array
     */

    public static function URIExplode($delimiter, $string) {
        $string_c = strlen($string);
        $string = substr($string, 1, $string_c);
        $dot = explode($delimiter, $string);
        return $dot;
    }

    /*
     * Filters Images
     */

    public static function IMGFilter($string) {
        $dot = self::htmlchars_decode($string);
        preg_match_all('!\[img\]((\[(.*?)\])*|.*?)\[\/img\]!ms', $dot, $imgs);
        foreach ($imgs[0] as $key => $val) {
            $dot = str_replace($val, '<img src="' . $imgs[1][$key] . '" border="0" alt="image">', $dot);
        }
        return $dot;
    }
    
    public static function getDataURI($image, $mime='') {
      return 'data: '.(function_exists('mime_content_type') ? mime_content_type($image) : $mime).';base64,'.base64_encode(file_get_contents($image));  
    }
    
    /*
     * Explode a uri string
     */

    public static function pagenameExplode($delimiter, $string) {
        $dot = str_replace($_SERVER['QUERY_STRING'], "", $string);
        $dot = explode($delimiter, $dot);
        return $dot;
    }
    
    public static function WordTruncate($input, $numWords) {
        if(str_word_count($input,0)>$numWords) {
            $WordKey = str_word_count($input,1);
            $WordIndex = array_flip(str_word_count($input,2));
            return substr($input,0,$WordIndex[$WordKey[$numWords]]);
        } else {
            return $input;

        }
    } 
    
    /*
     * Explode a queryString and return to global _POST
     */

    public static function queryStringExplode($delimiter, $string) {
        $dot = explode($delimiter, $string);
        if (is_array($dot) && !empty($dot)) {
            foreach ($dot as $key => $val) {
                $post = explode("=", $val);
                $_POST[$post[0]] = $post[1];
            }
        }
    }

    /*
     * Build options for drop down select
     * e. g. <option value="1">1</option>
     */
    public static function getOptions($array, $select, $style='', $standard='', $options='') {
        $n = "\n";
        if(is_array($array)) {
            
            foreach ($array as $key => $val) {
                $selected = ($select == $key && isset($select)) || (!empty($standard) && $standard == $key) ? ' selected="selected"' : '';
                $css = str_replace('###key###', $key, $style);
                $options .= '<option value="'. $key .'"'.$selected. $css .'>'. $val .'</option>'.$n;
            }
        }
        return $options;
    }
    
    public static function getChecked($array, $name, $value, $TT) {
        foreach($array as $key => $val) {
            $TT->setMC($name.$val, $val == $value ? ' checked="checked"' : '');
        }
    }
    
    public static function PrepareTime($time) {
        if($time != '') {
            $time = explode(":", $time);
            return sprintf("%'.02d",$time[0]).':'. sprintf("%'.02d", $time[1]) .':'. sprintf("%'.02d", $time[2]);
        } else {
            return 'NULL';
        }
    }
    
    public static function InsertValueIntoArrayPosition($array, $position, $value) {
        $a = 0;
        if(is_array($array)) {
            foreach($array as $key => $val) {
                if(is_int($key)) {
                    if($key == $position) {
                        $NewArray[] = $value;
                    }
                    $NewArray[] = $val;
                } else {
                    if($a == $position) {
                        if(is_array($value))
                            foreach($value as $vkey => $vval) {
                                $NewArray[$vkey] = $vval;
                            }
                    }
                    $NewArray[$key] = $val;
                }
                $a++;
            }
            return $NewArray;
        }
    }
    
    /**
     * Get date in sql format
     * @param string $date
     * @return string
     */
    public static function getSQLDate($date) {
        if($date != '') {
            $date = explode('.', $date);
            return $date[2].'-'.$date[1].'-'.$date[0];
        } else {
            return '0000-00-00';
        }
    }
    
    /*
     * Explode a encode queryString and return to global _POST
     */

    public static function queryStringDecodeExplode($delimiter, $string) {
        $dot_b64 = base64_decode($string);
        if (!empty($dot_b64)) {
            $dot = explode($delimiter, $dot_b64);
            if(base64_decode($string, TRUE) == TRUE) {
                if (is_array($dot) && !empty($dot)) {
                    foreach ($dot as $key => $val) {
                        $post = explode("=", $val);
                        $_POST[$post[0]] = $post[1];
                    }
                }
            } else {
                self::queryStringExplode($delimiter, $string);
            }
        } else {
            self::queryStringExplode($delimiter, $string);
        }
    }

    /*
     * Explodes the SESSION_ID from uri
     */

    public static function sidExplode($delimiter, $string) {
        $dot = explode($delimiter, $string);
        return $dot[2];
    }

    /*
     * Explodes the Language from uri
     */

    public static function langExplode($delimiter, $string) {
        $dot = explode($delimiter, $string);
        return $dot[2];
    }

    /*
     * Explodes the GET Parameters from pagename
     */

    public static function getExplode($delimiter, $string) {
        $dot = substr_count($string, $delimiter);
        if ($dot >= 1) {
            $dot = explode($delimiter, $string);
            return $dot;
        } else {
            return false;
        }
    }
    
    /**
     * Gender options
     */
    public static function Gender($selected, $options='') {
        if(isset($GLOBALS['GENDER']) && is_array($GLOBALS['GENDER'])) {
            foreach($GLOBALS['GENDER'] as $key => $val) {
                $select = $key == $selected ? ' selected="selected"' : ''; 
                $options .= '<option value="'.$key.'"'.$select.'>'.$val.'</option>';
            }
        }
        return $options;
    }
    
    public static function GetFloatArray($array, $array_names) {
        foreach($array as $key => $val) {
            if(!in_array($key, $array_names)) {
                $array[$key] = str_replace(',', '.', $val);
            }
        }
        return $array;        
    }
    
    
    /*
     * Read a $file into a string = $buffer and returns it.
     */

    public static function readfiles($file, $ext = false) {
        $verz = $ext === false ? DOC_ROOT : "";
        $fp = @fopen($verz . $file, 'r');
        if ($fp) {
            while (!feof($fp)) {
                $buffer.=fgets($fp);
            }
            fclose($fp);
        } else {
            $buffer = "Datei konnte nicht ge&ouml;ffnet werden bzw. Datei existiert nicht<br />Falsche Pfadangabe:" . $verz . $file;
        }
        return $buffer;
    }
    
    /**
     * Check if the directory exists. If not than creat it
     * @param string $dir
     */
    public static function CheckIfDirectoryExists($dir) {
        if($dir != '' && !empty($dir)) {
            if(file_exists($dir)) {
                return true;
            } else {
                if(self::CreatDirectory($dir)) {
                    return true;
                } else {
                    return false;
                }
            }
        } else {
            return false;
        }
    }
    
    /**
     * Creat a directory
     * @param string $dir
     */
    public static function CreatDirectory($dir) {
        $dir_array = explode("/", trim($dir, "/"));
        if(is_array($dir_array)) {
            $folder = "/";
            foreach($dir_array as $key => $val) {
                $slash = $key != 0 ? "/" : "";
                $folder .= $slash.$val;
                if(!file_exists(DOC_ROOT.$folder)) {
                    mkdir(DOC_ROOT . $folder);
                    chmod(DOC_ROOT . $folder, 0755);
                }    
            } 
            return true;
        } else {
            return false;
        }
    }

    /*
     * Process to avoid manipulation by login
     * ++ Username
     *
     */

    public static function avUser($username) {
        $dot = trim($username);
        $dot = strip_tags($dot);
        return $dot;
    }

    /*
     * Process for prices e.g 3.00 Euro
     *
     */

    public static function makePrice($price) {
        $price = explode(".", round($price, 2));
        if (is_array($price)) {
            $price_comma = strlen($price[1]) > 1 ? $price[1] : $price[1] * 10;
            $price = sprintf("%6d", $price[0]) . "." . sprintf("%02d", substr($price_comma, 0, 2));
        } else {
            $price = sprintf("%6d.00", $price);
        }
        return $price;
    }

    /*
     * Process for uri name e.g home.html not this höme.html
     *
     */

    public static function updatePagename($name, $lang = 'de') {
        $name = mb_strtolower($name, 'UTF-8');
        $name = Functions::mb_trim($name);

        $name = str_replace("ä", "ae", $name);
        $name = str_replace("ö", "oe", $name);
        $name = str_replace("ü", "ue", $name);
        $name = str_replace("ß", "ss", $name);
        $name = str_replace(" ", "_", $name);
        $name = str_replace("&", "_", $name);
        $name = str_replace("-", "_", $name);
        $name = str_replace("„", "", $name);
        $name = str_replace("“", "", $name);
        $name = str_replace("'", "", $name);
        $name = str_replace("´", "", $name);
        $name = str_replace("@", "", $name);
        $name = str_replace("¡", "", $name);
        $name = str_replace("]", "", $name);
        $name = str_replace("[", "", $name);
        $name = str_replace("«", "", $name);
        $name = str_replace("®", "", $name);
        $name = str_replace("©", "", $name);
        $name = str_replace("=", "", $name);
        $name = str_replace("€", "", $name);
        $name = str_replace("²", "", $name);
        $name = str_replace("±", "", $name);
        $name = str_replace("«", "", $name);
        $name = str_replace("∑", "", $name);
        $name = str_replace("†", "", $name);
        $name = str_replace("≈", "", $name);
        $name = str_replace("≠", "", $name);
        $name = str_replace("°", "", $name);
        if ($lang == "de") {
            if (preg_match_all('/\w/', $name, $nameArr)) {
                $name = '';
                foreach ($nameArr[0] as $key => $val) {
                    $name .= str_replace("_", "-", $val);
                }
                $name = preg_replace('/-$/', "", $name);
            }
            $name = str_replace("—", "", $name);
        } else {
            $name = str_replace("_", "-", $name);
            $name = str_replace(":", "", $name);
            $name = str_replace("+", "", $name);
            $name = str_replace("$", "", $name);
            $name = str_replace("!", "", $name);
            $name = str_replace('"', "", $name);
            $name = str_replace("'", "", $name);
            $name = str_replace("*", "", $name);
            $name = str_replace("#", "", $name);
            $name = str_replace("´", "", $name);
            $name = str_replace("`", "", $name);
            $name = str_replace("^", "", $name);
            $name = str_replace("°", "", $name);
            $name = str_replace("/", "-", $name);
            $name = str_replace("%", "", $name);
            $name = str_replace(".", "", $name);
            $name = str_replace(";", "", $name);
            $name = str_replace(",", "", $name);
            $name = str_replace("—", "", $name);
            $name = str_replace("<", "", $name);
            $name = str_replace(">", "", $name);
            $name = str_replace("?", "", $name);
            $name = str_replace("[", "", $name);
            $name = str_replace("]", "", $name);
            $name = str_replace("€", "", $name);
            $name = str_replace("²", "", $name);
        }
        return $name;
    }

    public static function mb_trim($string) {
        $string = preg_replace("/(^\s+)|(\s+$)/us", "", $string);
        return $string;
    }

    public static function xmlCharsDecode($string) {
        $dot = str_replace("&#228;", "ä", $string);
        $dot = str_replace("&#246;", "ö", $dot);
        $dot = str_replace("&#252;", "ü", $dot);
        $dot = str_replace("&#196;", "Ä", $dot);
        $dot = str_replace("&#220;", "Ü", $dot);
        $dot = str_replace("&#214;", "Ö", $dot);
        $dot = str_replace("&#223;", "ß", $dot);
        $dot = str_replace("&#171;", "«", $dot);
        $dot = str_replace("&#187;", "»", $dot);
        $dot = str_replace("&#38;", "&", $dot);
        $dot = str_replace("&#214;", "%", $dot);
        $dot = str_replace("&#223;", "$", $dot);
        $dot = str_replace("&#232;", "è", $dot);
        $dot = str_replace("&#233;", "é", $dot);
        $dot = str_replace("&#167;", "§", $dot);

        return $dot;
    }

    public static function removeCriticSigns($string) {
        $dot = strip_tags(stripslashes(trim($string)));
        $dot = str_replace('"', "&quot;", $dot);
        $dot = str_replace("'", "&quot;", $dot);
        $dot = str_replace("<", "&lt;", $dot);
        $dot = str_replace(">", "&gt;", $dot);
        return $dot;
    }

    public static function replaceBBCode($string, $color) {
        $string = str_replace("[i]", "<em>", $string);
        $string = str_replace("[/i]", "</em>", $string);
        $string = str_replace("[b]", "<strong>", $string);
        $string = str_replace("[/b]", "</strong>", $string);
        $string = str_replace("[u]", "<u>", $string);
        $string = str_replace("[/u]", "</u>", $string);
        $string = str_replace("[img]", "", $string);
        $string = str_replace("[/img]", "", $string);
        $string = str_replace("[url]", "", $string);
        $string = str_replace("[/url]", "", $string);
        $string = str_replace("[/color]", "</font>", $string);
        if (is_array($color) && !empty($color)) {
            foreach ($color as $key => $val) {
                $string = str_replace("[color=" . $val . "]", "<font color=\"" . $val . "\">", $string);
            }
        }
        return $string;
    }

    public static function decodeCriticSigns($string) {
        $dot = str_replace("&quot;", '"', $string);
        $dot = str_replace("&#039;", "'", $dot);
        $dot = str_replace("&lt;", "<", $dot);
        $dot = str_replace("&gt;", ">", $dot);
        return $dot;
    }

    /*
     * Inverse version of htmlspecialchars()
     */

    public static function htmlchars_decode($val) {
        $dot = str_replace("&amp;", "&", $val);
        $dot = str_replace("&gt;", ">", $dot);
        $dot = str_replace("&lt;", "<", $dot);
        $dot = str_replace("&quot;", '"', $dot);
        $dot = str_replace("&#039;", "'", $dot);
        return $dot;
    }

    public static function htmlchars_encode($val) {
        $dot = str_replace("&", "&amp;", $val);
        $dot = str_replace(">", "&gt;", $dot);
        $dot = str_replace("<", "&lt;", $dot);
        $dot = str_replace('"', "&quot;", $dot);
        $dot = str_replace("'", "&#039;", $dot);
        return $dot;
    }
    
    /*
     * Encrypt E-Mail-Addresses
     */

    public static function encrypt_email($val) {
        $dot = str_replace("i", "&#105;", $val);
        $dot = str_replace("a", "&#97;", $dot);
        $dot = str_replace("o", "&#111;", $dot);
        $dot = str_replace("e", "&#101;", $dot);
        $dot = str_replace("u", "&#117;", $dot);
        $dot = str_replace("@", "&#64;", $dot);
        $dot = str_replace(".", "&#46;", $dot);
        return $dot;
    }

    /*
     * Umlaute
     */

    public static function umlaute($string, $strto) {
        switch ($strto) {
            case "upper":
                $dot = str_replace("ü", "&Uuml;", $string);
                $dot = str_replace("Ü", "&Uuml;", $dot);
                $dot = str_replace("ä", "&Auml;", $dot);
                $dot = str_replace("Ä", "&Auml;", $dot);
                $dot = str_replace("ö", "&Ouml;", $dot);
                $dot = str_replace("Ö", "&Ouml;", $dot);
                break;
            case "lower":

                break;
        }
        return $dot;
    }

    public static function removeElementMarker($html) {
        if (preg_match_all('!marker\="(.*?)"!sim', $html, $marker)) {
            foreach ($marker[0] as $key => $val) {
                $html = str_replace($val, $marker[1][$key], $html);
            }
        }
        return $html;
    }

    public static function blankUID($string) {
        return intval(str_replace("#", "", str_replace("dotlink-", "", $string)));
    }

    /*
     * Mail functions
     *
     *
     */

    public static function ImageSCRAbsolut($count, $array, $replace, $content, $a = 0) {
        for ($i = $a; $i < $count; $i++) {
            $content = str_replace($array[$i], $replace[$i], $content);
        }
        return $content;
    }

    public static function replaceImageSRCToHost($bodytext) {

        if (preg_match_all('#<img*.src=\"(.*?)\"[^>]+>#ism', $bodytext, $tmp_images)) {
            foreach ($tmp_images[1] as $key => $val) {
                if (!preg_match("#(http:|https:)#", $val, $erfolg)) {
                    $img_src = HTTPO . $val;
                    $bodytext = preg_replace('!"' . $val . '"!', '"' . $img_src . '"', $bodytext);
                }
            }
        }

        # Background Images tauschen
        if (preg_match_all('#url\((.*?)\)#ism', $bodytext, $bgimages)) {
            foreach ($bgimages[1] as $key => $val) {
                $bodytext = str_replace($val, HTTPO . $val, $bodytext);
            }
        }
        return $bodytext;
    }

    /*
     * Menu functions
     *
     *
     */

    public static function breadcrumb($puid, $classObj = "dotFEMenu", $lang = "") {
        if ($classObj == "dotFEMenu") {
            $pfad = \lib\fe_menu::getTreePathNames($puid, '', 0, 1, 0, $lang);
        } else {
            $pfad = \lib\be_menu::getTreePathNames($puid, '', 0, 1, 0, $lang);
        }
        $pfad = explode("/", str_replace(" / ", " # ", str_replace("w/m", "#wm#", $pfad)));

        $x = 1;
        foreach ($pfad as $key => $val) {
            $pfeil = count($pfad) == $x ? "" : " > ";
            if (!empty($val)) {
                if ($val != '-0-') {
                    $pfad_n .= str_replace(" # ", " / ", str_replace("#wm#", "w/m", $val)) . $pfeil;
                }
            }
            $x++;
        }

        return $pfad_n;
    }

    public static function breadcrumbLink($puid, $trenner = "&gt;") {
        $db = \lib\Registry::getObj('db');

        if ($GLOBALS['LANGUAGE'] != SYS_LANGUAGE) {
            $Pages = "pages_language";
            $and_lang = "AND %%%n%%%lang = :lang ";
            $lang = "/" . $GLOBALS['LANGUAGE'];
        } else {
            $Pages = "pages";
            $and_lang = "";
        }

        $db->Select("p.uid, p.pagename, p.nav_title, p.lft, p.rgt, p.urltype, p.url, n.lft AS nlft")
           ->From($Pages . " n, fe_" . $Pages . " p")
           ->noWhere(1)
           ->Andd("WHERE n.lft BETWEEN p.lft AND p.rgt AND n.uid = :uid	" . str_replace("%%%n%%%", "p.", $and_lang))
           ->Order("ORDER BY p.lft")
           ->Parameter($and_lang != '' ? array('uid' => intval($puid), 'lang' => $GLOBALS['LANGUAGE']) : array('uid' => intval($puid)))
           ->SelectMethod(1)
           ->doSelect();
        
        $tmp_tree = ''; # Cont Buffer;
        $num = $db->Rows;
        $x = 1;
        $link_buffer = "";
        if($num) {
            foreach ($db->SelectPuffer as $key => $row) {
                $sep = $num != $x ? $trenner . " " : "";
                if ($row['urltype'] == 0) {
                    $pagelink = self::dotLink($row['uid'], $GLOBALS['LANGUAGE']);
                    $breadcrumb .= "<a href=\"" . $pagelink . "\">" . $row['nav_title'] . "</a> " . $sep;
                } elseif ($row['urltype'] == 4) {
                    $jslink = $row['url'];
                    $breadcrumb .= "<a href=\"javascript:void(0);\" onclick=\"javascript: " . $jslink . "\">" . $row['nav_title'] . "</a> " . $sep;
                } elseif ($row['urltype'] == 5) {
                    $pagelink = self::dotLink($row['url'], $GLOBALS['LANGUAGE']);
                    $breadcrumb .= "<a href=\"" . $pagelink . "\">" . $row['nav_title'] . "</a> " . $sep;
                } else {
                    $pagelink = $link_buffer . "/" . $row['pagename'];
                    $breadcrumb .= "<a href=\"" . $lang . $pagelink . "\">" . $row['nav_title'] . "</a> " . $sep;
                }

                $link_buffer = $pagelink;
                $x++;
            }
        }
        return $breadcrumb;
    }

    public static function dotLink($link, $lang = '') {
        if (preg_match('%dotlink%', $link, $treffer)) {
            $dotlink = explode("dotlink-", str_replace("#", "", $link));
        } else {
            $dotlink[1] = $link;
        }
        return \lib\fe_menu::getTreePathNames(intval($dotlink[1]), '', 0, 1, 1, $lang, ' AND p.docType != 0 ');
    }

    public static function dotLinkBE($link, $lang = '') {
        if (preg_match('%dotlink%', $link, $treffer)) {
            $dotlink = explode("dotlink-", str_replace("#", "", $link));
        } else {
            $dotlink[1] = $link;
        }
        
        return ARIBU_SYSTEMS_URL . \lib\be_menu::getTreePathNames(intval($dotlink[1]), '', 0, 1, 1, $lang);
        
    }

    public static function replaceLinks($html, $linkMethod = 'dotLink') {
        $new_html = self::replaceLinksforEMail($html, $linkMethod, 2);
        
        return self::replaceLinksNew($new_html, $linkMethod);
    }
    
    public static function replaceLinksNew($html, $linkMethod = 'dotLink') {
        if(preg_match_all('/###dotlink-(.*)###/im', $html, $matches, PREG_SET_ORDER, 0)) {
            if(is_array($matches)) {
                foreach($matches as $key => $val) {
                    $html = str_replace($val[0], self::$linkMethod($val[1], $GLOBALS['LANGUAGE']), $html);
                }
            }
        }
        return $html;
    }

    public static function replaceLinksforEMail($bodytext, $linkMethod = 'dotLink', $flag = 1) {
        # Links ersetzen href
        if($flag == 1) {
            $protocol_url = URL_PROTOCOL == 'https'  ? 'https://'.URL_HOST : 'http://'.URL_HOST; 
        }
        if (preg_match_all('/href="([^"]*)"/', $bodytext, $links)) {
            $bodytext = self::ReplaceLinksWithBodytext($bodytext, $links, $protocol_url, $linkMethod, 'href');
        } 
        unset($links);
        if(preg_match_all('/action="([^"]*)"/', $bodytext, $links)) {
            $bodytext = self::ReplaceLinksWithBodytext($bodytext, $links, $protocol_url, $linkMethod, 'action');
        }            
        return $bodytext;
    }
    
    public static function ReplaceLinksWithBodytext($bodytext, $links, $protocol_url, $linkMethod, $tag) {
        $links_array = array_unique($links[0]);
        $links_array_raw = array_unique($links[1]);
        $il = array();
        $nval = '';
        foreach ($links_array as $key => $val) {
            if(!preg_match("#(http|https|ftp|dotlink|mailto)#", $links_array_raw[$key])) {
                $bodytext = str_replace($val, $tag.'="'.$protocol_url.$links_array_raw[$key].'"', $bodytext);
            } else {
                if(preg_match("#dotlink#", $links_array_raw[$key])) {
                    # Interne Links ersetzen
                    if(preg_match('/#{0,3}dotlink-[a-z]{0,2}[0-9]{1,8}#{0,3}(.*)/', $links_array_raw[$key], $il)) {
                        $nval = str_replace($il[1],'',$links_array_raw[$key]);
                    }
                    $internerlink = self::$linkMethod($nval, $GLOBALS['LANGUAGE']);
                    $bodytext = str_replace($val, $tag.'="'.$protocol_url.$internerlink.$il[1].'"', $bodytext);
                }
            }
        }
        return $bodytext;
    }

    /*
     * File functions
     *
     *
     */

    public static function fixUploadName($name, $lang = 'de') {
        $name = self::mb_trim($name);
                  
        if ($lang == "de") {
            $name = str_replace("ä", "ae", $name);
            $name = str_replace("ö", "oe", $name);
            $name = str_replace("ü", "ue", $name);
            $name = str_replace("ß", "ss", $name);
        }
        $name = str_replace(" ", "_", $name);
        $name = str_replace("&", "_", $name);
        $name = str_replace("-", "_", $name);
        $name = str_replace("„", "", $name);
        $name = str_replace("“", "", $name);
        $name = str_replace("'", "", $name);
        $name = str_replace("´", "", $name);
        $name = str_replace("@", "", $name);
        $name = str_replace("¡", "", $name);
        $name = str_replace("]", "", $name);
        $name = str_replace("[", "", $name);
        $name = str_replace("«", "", $name);
        $name = str_replace("®", "", $name);
        $name = str_replace("©", "", $name);
        $name = str_replace("=", "", $name);
        if ($lang == "de") {
            if (preg_match_all('/\w/', $name, $nameArr)) {
                $name = '';
                foreach ($nameArr[0] as $key => $val) {
                    $name .= str_replace("_", "-", $val);
                }
                $name = preg_replace('/-$/', "", $name);
            }
            $name = str_replace("—", "", $name);
        } else {
            $name = str_replace("_", "-", $name);
            $name = str_replace(":", "", $name);
            $name = str_replace("+", "", $name);
            $name = str_replace("$", "", $name);
            $name = str_replace("!", "", $name);
            $name = str_replace('"', "", $name);
            $name = str_replace("'", "", $name);
            $name = str_replace("*", "", $name);
            $name = str_replace("#", "", $name);
            $name = str_replace("´", "", $name);
            $name = str_replace("`", "", $name);
            $name = str_replace("^", "", $name);
            $name = str_replace("°", "", $name);
            $name = str_replace("/", "-", $name);
            $name = str_replace("%", "", $name);
            $name = str_replace(".", "", $name);
            $name = str_replace(";", "", $name);
            $name = str_replace(",", "", $name);
            $name = str_replace("—", "", $name);
            $name = str_replace("<", "", $name);
            $name = str_replace(">", "", $name);
            $name = str_replace("?", "", $name);
            $name = str_replace("[", "", $name);
            $name = str_replace("]", "", $name);
        }
        return $name;
    }

    public static function UploadFileName($FileName) {
        $FileName = explode(".", $FileName);
        return $FileName[0];
    }

    public static function getDirs($dir, $nDir = "/", $dirs=array()) {        
        if ($handle = opendir($dir)) {
            while (false !== ($file = readdir($handle))) {
                if (filetype($dir . $file) === 'dir' && $file != "." && $file != "..") {
                    array_push($dirs, $nDir . $file);
                    $dirs = self::getDirs($dir . $file . "/", $nDir . $file . "/", $dirs);
                }
            }
            closedir($handle);
        }
        
        return $dirs;
    }

    public static function readDirectory($dir, $dir_link = '', $withdir = 1) {
        $dirnames = array();
        $fp = opendir($dir);
        while ($file = readdir($fp)) {
            if ($file != '.' && $file != '..' && is_file($dir . "/" . $file)) {
                $dirnames[] = $file;
            } else {
                if ($withdir == 1) {
                    if ($file != '.' && $file != '..') {
                        $folder = $_GET['ordner'] != "" ? $_GET['ordner'] . "/" : "";
                        $dirnames[] = "<a href=\"" . $dir_link . "&ordner=" . $folder . $file . "\">" . $file . "</a>";
                    }
                }
            }
        }
        closedir($fp);
        sort($dirnames);
        return $dirnames;
    }
    
    /**
     * Deletes a directory and all files and directories under it
     * @param string $dirPath
     * @throws \InvalidArgumentException
     */
    public static function deleteDir($dirPath) {
        if (!is_dir($dirPath)) {
            throw new \InvalidArgumentException($dirPath . " must be a directory");
        }
        
        
        if (substr($dirPath, strlen($dirPath) - 1, 1) != '/') {
            $dirPath .= '/';
        }
        $files = scandir($dirPath); 
        foreach ($files as $file) {
            if ($file === '.' || $file === '..') continue;
            if (is_dir($dirPath.$file)) {
                self::deleteDir($dirPath.$file);
            } else {
                if ($dirPath.$file !== __FILE__) {
                    unlink($dirPath.$file);
                }
            }
        }
        rmdir($dirPath);
    }

    public static function FileManagerStrings($dir, $next = '', $folder = '') {
        $handle = opendir($dir);
        static $a = 0;
        while ($file = readdir($handle)) {
            if ($file != '.' && $file != '..') { //Versteckte Dateien nicht anzeigen
                if (is_dir($dir . '/' . $file)) {
                    $folderArray[] = $file;
                } else {
                    if (!preg_match('#.LCK#', $file, $match)) {
                        $fileArray[] = $file;
                    }
                }
            }
        }
        closedir($handle);
        $n_folder = str_replace("/fileadmin/img/", "", $folder);
        if (isset($fileArray) && is_array($fileArray)) {
            asort($fileArray);
            foreach ($fileArray as $fkey => $fval) {
                $js_string .= "folderimages[" . $a . "] = new Array( '" . $n_folder . "','" . $fval . "','" . $fval . "' );\n";
                $a++;
            }
        }

        $nDir = str_replace($_SERVER['DOCUMENT_ROOT'], "", $dir);
        $nDir = str_replace("/fileadmin/img/", "", $nDir);

        # Ordner verarbeiten
        if (isset($folderArray) && is_array($folderArray)) {
            asort($folderArray);
            foreach ($folderArray as $key => $val) {
                #$folders .= "<option value=\"/".$next.$val."/\">/".$next.$val."</option>\n";
                $folder = $next . $val . "/";
                if ($tmp = self::FileManagerStrings($dir . '/' . $val, $nDir . "/" . $val . '/', $folder)) {
                    $js_string .= $tmp;
                }
            }
        }
        return $js_string;
    }

    public static function FileManagerOptions($dir, $next = 0, $folder = '') {
        $handle = opendir($dir);
        $nDir = str_replace($_SERVER['DOCUMENT_ROOT'], "", $dir);
        $nDir = str_replace("/fileadmin/img/", "", $nDir);
        if ($dir != IMAGES_ROOT) {
            $folders .= "<option value=\"" . $nDir . "/\">/" . $nDir . "</option>\n";
        }
        while ($file = readdir($handle)) {
            if (is_dir($dir . '/' . $file) && $file != '.' && $file != '..') { //Versteckte Dateien nicht anzeigen
                if ($tmp = self::FileManagerOptions($dir . "/" . $file, $a)) {
                    $folders .= $tmp;
                }
            }
        }
        closedir($handle);
        return $folders;
    }

    /*
     * Read File into array
     *
     */

    public static function readCSV($file, $ext = false) {
        $verz = $ext === false ? DOC_ROOT : "";
        $fp = @fopen($verz . $file, 'r');
        if ($fp) {
            while (!feof($fp)) {
                $buffer[] = fgets($fp);
            }
            fclose($fp);
        } else {
            $buffer = "Datei konnte nicht geöffnet werden bzw. Datei existiert nicht<br>Falsche Pfadangabe:" . $verz . $file;
        }
        return $buffer;
    }

    public static function csvString($string) {
        mb_internal_encoding("UTF-8");
        if (preg_match('/"/', $string, $arr)) {
            $string = str_replace("\n", "", $string);
            $string = mb_substr($string, 1);
            $string = mb_substr($string, 0, -1);
            $string = str_replace('""', '"', $string);
        }
        return $string;
    }

    public static function csvToArray($csv) {
        $a = 0;
        $csvArray = array();
        $row_names = array();
        if (is_array($csv)) {
            foreach ($csv as $key => $val) {
                if (mb_detect_encoding($val, 'UTF-8, ISO-8859-1') === 'UTF-8') {
                    $csv_row = explode(";", $val);
                } else {
                    $csv_row = explode(";", utf8_encode($val));
                }

                if ($a == 0) {
                    foreach ($csv_row as $nkey => $nval) {
                        $row_names[$nkey] = self::csvString($nval);
                    }
                } else {
                    foreach ($csv_row as $akey => $aval) {
                        $csvArray[$a - 1][$row_names[$akey]] = self::csvString($aval);
                    }
                }
                $a++;
            }
        }
        return $csvArray;
    }

    
    public static function CalcPages($max_items, $max_view = 10) {
        if ($max_items % $max_view == 0) {
            $pages = $max_items / $max_view;
        } else {
            $pages = floor($max_items / $max_view + 1);
        }
        
        return $pages;
    }
    
    /*
     * Blätter function
     *
     *
     */

    public static function scrolling($max_items = 0, $request = '', $uri = '/', $max_view = 10, $base = 1, $oLink = 1) {
        $amp = preg_match('#=#ism', $request) ? '&' : '';
        $http = \lib\Registry::getObj('http');
        
        if ($http->issetPost('SEITE') === false) {
            $http->setPostbyName('SEITE', 1);
        }
        if (intval($http->_post('SEITE')) < 0) {
            $http->setPostbyName('SEITE', 1);
        }
        if ($max_items > $max_view) {
            if ($max_items % $max_view == 0) {
                $pages = $max_items / $max_view;
            } else {
                $pages = floor($max_items / $max_view + 1);
            }
            if ($oLink == 1) {
                $uri1 = $base == '1' ? base64_encode($request . $amp. "SEITE=1") : $request . $amp . "SEITE=1";
                $uri2 = $base == '1' ? base64_encode($request . $amp. "SEITE=" . (intval($http->_post('SEITE')) - 1)) : $request . $amp. "SEITE=" . (intval($http->_post('SEITE')) - 1);
            } else {
                $uri1 = "1/" . $request;
                $uri2 = (intval($http->_post('SEITE')) - 1) . "/" . $request;
            }
            $url1 = $uri . $uri1;
            $url2 = $uri . $uri2;

            if (intval($http->_post('SEITE')) != 1)
                $page_link .= "<a class=\"search-scrolling\" href=\"" . $url1 . "\"><strong>&laquo;</strong></a>&nbsp;&nbsp;<a href=\"" . $url2 . "\" class=\"search-scrolling\"><b>&#8249;</b></a>";
            if (intval($http->_post('SEITE')) + 4 >= $pages) {
                $pagex = $pages;
            } else {
                $pagex = intval($http->_post('SEITE')) + 4;
            }

            for ($i = intval($http->_post('SEITE')) - 4; $i <= $pagex; $i++) {
                if ($i <= 0) {
                    $i = 1;
                }
                if ($oLink == 1) {
                    $uri3 = $base == '1' ? base64_encode($request . $amp . "SEITE=" . $i) : $request . $amp ."SEITE=" . $i;
                } else {
                    $uri3 = $i . "/" . $request;
                }
                $url3 = $uri . $uri3;
                if ($i == intval($http->_post('SEITE'))) {
                    $page_link .= "&nbsp;&nbsp;<strong>[" . $i . "]</strong>";
                } else {
                    $page_link .= "&nbsp;&nbsp;<a href=\"" . $url3 . "\" class=\"search-scrolling\">$i</a>";
                }
            }

            if ($oLink == 1) {
                $uri4 = $base == '1' ? base64_encode($request . $amp . "SEITE=" . (intval($http->_post('SEITE')) + 1)) : $request . $amp . "SEITE=" . (intval($http->_post('SEITE')) + 1);
                $uri5 = $base == '1' ? base64_encode($request . $amp . "SEITE=" . $pages) : $request . $amp . "SEITE=" . $pages;
            } else {
                $uri4 = (intval($http->_post('SEITE')) + 1) . "/" . $request;
                $uri5 = $pages . "/" . $request;
            }
            $url4 = $uri . $uri4;
            $url5 = $uri . $uri5;
            if (intval($http->_post('SEITE')) != $pages)
                $page_link .= "&nbsp;&nbsp;<a href=\"" . $url4 . "\" class=\"search-scrolling\"><b>&rsaquo;</b></a>&nbsp;&nbsp;<a class=\"search-scrolling\" href=\"" . $url5 . "\"><strong>&raquo;</strong></a>";
            $dot = $page_link;
            return $dot;
        } else {
            return false;
        }
    }

    /*
     * Debug modus
     *
     *
     */

    public static function debug($var = '', $global = '') {
        echo "<font color=\"#333333\" size=\"3\" face=\"Verdana, Arial, Helvetica, sans-serif\"><pre>";
        if (!empty($global)) {
            switch ($global) {
                case 'POST': print_r($GLOBALS['POST']);
                    break;
                case 'GET': print_r($GLOBALS['GET']);
                    break;
                case 'REQUEST': print_r($GLOBALS[_REQUEST]);
                    break;
                case 'COOKIE': print_r($GLOBALS[$HTTP_COOKIE_VARS]);
                    break;
                case 'SERVER': print_r($GLOBALS[$HTTP_SERVER_VARS]);
                    break;
                case 'SESSION': print_r($GLOBALS[_SESSION]);
                    break;
                case 'GLOBALS': print_r($GLOBALS);
                    break;
                case 'FILES': print_r($_FILES);
                    break;
            }
        } else {
            print_r($var);
        }
        echo "</pre></font>";
    }

    /*
     * Community functions
     *
     *
     */

    public static function age($day, $month, $year) {

        $time = time();
        $current_day = date("j", $time);
        $current_month = date("n", $time);
        $current_year = date("Y", $time);

        $age_year = $current_year - $year;

        if (($current_month == $month && $current_day < $day) || $current_month < $month) {
            $age = $age_year - 1;
        } else {
            $age = $age_year;
        }
        return $age;
    }

    /*
     * XML2Array
     *
     *
     */

    public static function xml2array($contents, $get_attributes = 1, $priority = 'tag') {
        if (!$contents)
            return array();

        if (!function_exists('xml_parser_create')) {
            //print "'xml_parser_create()' function not found!"; 
            return array();
        }

        //Get the XML parser of PHP - PHP must have this module for the parser to work 
        $parser = xml_parser_create('');
        xml_parser_set_option($parser, XML_OPTION_TARGET_ENCODING, "UTF-8"); # http://minutillo.com/steve/weblog/2004/6/17/php-xml-and-character-encodings-a-tale-of-sadness-rage-and-data-loss 
        xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
        xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
        xml_parse_into_struct($parser, trim($contents), $xml_values);
        xml_parser_free($parser);

        if (!$xml_values)
            return; //Hmm... 

            
        //Initializations 
        $xml_array = array();
        $parents = array();
        $opened_tags = array();
        $arr = array();

        $current = &$xml_array; //Refference 
        //Go through the tags. 
        $repeated_tag_index = array(); //Multiple tags with same name will be turned into an array 
        foreach ($xml_values as $data) {
            unset($attributes, $value); //Remove existing values, or there will be trouble 
            //This command will extract these variables into the foreach scope 
            // tag(string), type(string), level(int), attributes(array). 
            extract($data); //We could use the array by itself, but this cooler. 

            $result = array();
            $attributes_data = array();

            if (isset($value)) {
                if ($priority == 'tag')
                    $result = $value;
                else
                    $result['value'] = $value; //Put the value in a assoc array if we are in the 'Attribute' mode 
            }

            //Set the attributes too. 
            if (isset($attributes) and $get_attributes) {
                foreach ($attributes as $attr => $val) {
                    if ($priority == 'tag')
                        $attributes_data[$attr] = $val;
                    else
                        $result['attr'][$attr] = $val; //Set all the attributes in a array called 'attr' 
                }
            }

            //See tag status and do the needed. 
            if ($type == "open") {//The starting of the tag '<tag>' 
                $parent[$level - 1] = &$current;
                if (!is_array($current) or ( !in_array($tag, array_keys($current)))) { //Insert New tag 
                    $current[$tag] = $result;
                    if ($attributes_data)
                        $current[$tag . '_attr'] = $attributes_data;
                    $repeated_tag_index[$tag . '_' . $level] = 1;

                    $current = &$current[$tag];
                } else { //There was another element with the same tag name 
                    if (isset($current[$tag][0])) {//If there is a 0th element it is already an array 
                        $current[$tag][$repeated_tag_index[$tag . '_' . $level]] = $result;
                        $repeated_tag_index[$tag . '_' . $level] ++;
                    } else {//This section will make the value an array if multiple tags with the same name appear together 
                        $current[$tag] = array($current[$tag], $result); //This will combine the existing item and the new item together to make an array 
                        $repeated_tag_index[$tag . '_' . $level] = 2;

                        if (isset($current[$tag . '_attr'])) { //The attribute of the last(0th) tag must be moved as well 
                            $current[$tag]['0_attr'] = $current[$tag . '_attr'];
                            unset($current[$tag . '_attr']);
                        }
                    }
                    $last_item_index = $repeated_tag_index[$tag . '_' . $level] - 1;
                    $current = &$current[$tag][$last_item_index];
                }
            } elseif ($type == "complete") { //Tags that ends in 1 line '<tag />' 
                //See if the key is already taken. 
                if (!isset($current[$tag])) { //New Key 
                    $current[$tag] = $result;
                    $repeated_tag_index[$tag . '_' . $level] = 1;
                    if ($priority == 'tag' and $attributes_data)
                        $current[$tag . '_attr'] = $attributes_data;
                } else { //If taken, put all things inside a list(array) 
                    if (isset($current[$tag][0]) and is_array($current[$tag])) {//If it is already an array... 
                        // ...push the new element into that array. 
                        $current[$tag][$repeated_tag_index[$tag . '_' . $level]] = $result;

                        if ($priority == 'tag' and $get_attributes and $attributes_data) {
                            $current[$tag][$repeated_tag_index[$tag . '_' . $level] . '_attr'] = $attributes_data;
                        }
                        $repeated_tag_index[$tag . '_' . $level] ++;
                    } else { //If it is not an array... 
                        $current[$tag] = array($current[$tag], $result); //...Make it an array using using the existing value and the new value 
                        $repeated_tag_index[$tag . '_' . $level] = 1;
                        if ($priority == 'tag' and $get_attributes) {
                            if (isset($current[$tag . '_attr'])) { //The attribute of the last(0th) tag must be moved as well 
                                $current[$tag]['0_attr'] = $current[$tag . '_attr'];
                                unset($current[$tag . '_attr']);
                            }

                            if ($attributes_data) {
                                $current[$tag][$repeated_tag_index[$tag . '_' . $level] . '_attr'] = $attributes_data;
                            }
                        }
                        $repeated_tag_index[$tag . '_' . $level] ++; //0 and 1 index is already taken 
                    }
                }
            } elseif ($type == 'close') { //End of tag '</tag>' 
                $current = &$parent[$level - 1];
            }
        }

        return($xml_array);
    }
    
    /**
     * Recursive copy of folders and files
     * @param string $src
     * @param string $dst
     */
    public static function rcopy($src,$dst) {
	$dir = opendir($src);
        if(self::CheckIfDirectoryExists(str_replace(DOC_ROOT, '', $dst))) {}
	while(( $file = readdir($dir)) ) {
            if (( $file != '.' ) && ( $file != '..' ) && ($file != '')) {
                if ( is_dir($src . '/' . $file) ) {
                    if($dst != $src . '/' . $file) {
                        self::rcopy($src .'/'. $file, $dst .'/'. $file);
                    }
                } else {
                    copy($src .'/'. $file, $dst .'/'. $file);
                }
            }
	}
	closedir($dir);
    }
}