<?php

/* 
 * aribus systems - Version 7.0.0
 * 
 * Copyright (C) 2018  Sascha Momcilovic
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 3 of the License, or (at your option) 
 * any later version. This program is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, see <http://www.gnu.org/licenses/>. 
 * 

 * 
 * @author:  Sascha Momcilovic
 * @e-Mail:  info@it-orange.de
 * @link:    http://www.it-orange.de
 * @date:    02.04.2015 
 * @update:  08.04.2015
 */

class HttpRequest {
    
    /*
     * Post
     * @var string
     */
    private $post = array();

    /*
     * Cookie
     * @var string
     */
    private $cookie = array();
    
    /*
     * Request_Uri
     * e. g. /this/one/request/uri
     * @var string
     */
    private $Request_Uri;
    
    /**
     * Query string array
     * @var array
     */
    private $QueryString = array();
    

    public function __construct() {
        
        if((isset($_SERVER['QUERY_STRING']) && !empty($_SERVER['QUERY_STRING'])) || (isset($_SERVER['REQUEST_URI']) && !empty($_SERVER['REQUEST_URI']))) {
            $this->setQSBase64();
        }
        
        if(isset($_POST)) {
            $this->setPost($_POST);
        }
        
        if(isset($_GET)) {
            $this->setGet($_GET);
        }
        
        if(isset($_COOKIE)) {
            $this->setCookie($_COOKIE);
        }
        
        $this->setRequestUri();
        $this->setQueryStringArray();
    }
    
    public function setPost($arr) {
        if(is_array($arr)) {
            foreach($arr as $key => $value) {
                $this->post[$key] = $value;    
            }
        }
    }
    
    public function setPostbyName($name, $value) {
        if(is_array($value)) {
            foreach($value as $key => $val) {
               $this->post[$name][$key] = $val;  
            }
        } else {
            $this->post[$name] = $value;
        }
        return $this;
    }
    
    public function setGet($arr) {
        $this->setPost($arr);
    }
    
    public function setCookie($arr) {
        
    }
    
    public function setQueryStringArray() {
        if (base64_encode(base64_decode($_SERVER['QUERY_STRING'], true)) === $_SERVER['QUERY_STRING']) {
            
        } else {
            $qs_explode = explode("&", $_SERVER['QUERY_STRING']);
            if(is_array($qs_explode)) {
                foreach($qs_explode as $key => $val) {
                    $qs_arr = explode("=", $val);
                    if(is_array($qs_arr)) {
                        $this->addQS($qs_arr[0], $qs_arr[1]);
                    }
                }           
            }
        }
    }
    
    public function addQS($name, $value) {
        $this->QueryString[$name] = $value;
        return $this;
    }
    
    public function addQSPost($name) {
        if($this->issetPost($name)===true) {
            $this->QueryString[$name] = $this->_post($name);
        } 
        return $this;
    }
    
    public function unsetQS($name) {
        unset($this->QueryString[$name]);
        return $this;
    }
    
    public function getQS($name) {
        if(isset($this->QueryString[$name])) {
            return $this->QueryString[$name];
        } else {
            return false;
        }
    }

    public function returnQS($flag=NULL) {
        if(is_array($this->QueryString)) {
            foreach($this->QueryString as $key => $val) {
                $QueryStringArr[] = $key."=".$val;
            }
        }
        $qs = $flag === NULL ? '?' : ($flag == 2 ? '&' : '');
        $QueryString = is_array($QueryStringArr) ? $qs.implode('&', $QueryStringArr) : '';
        return $QueryString;
    }
    
    public function clearQS() {
        unset($this->QueryString);
        $this->QueryString = array();
        $this->setQueryStringArray();
        return $this;
    }
    
    public function setQSBase64() {
        if(!empty($_SERVER['QUERY_STRING'])) {
            \lib\Functions::queryStringDecodeExplode("&", $_SERVER['QUERY_STRING']);
	} elseif(strstr($_SERVER['REQUEST_URI'], "?")) {
            $query_string = explode("?", $_SERVER['REQUEST_URI']);
            \lib\Functions::queryStringDecodeExplode("&", $query_string[1]);
	}
    }
    
    public function setRequestUri() {
        $request_uri = urldecode($_SERVER['REQUEST_URI']);
        if (preg_match('/\?/ism',$request_uri)) {
            $request_uri_explode = explode("?", $request_uri);
            $this->Request_Uri = $request_uri_explode[0];
        } else {
            $this->Request_Uri = $request_uri;
        }
    }


    public function _post($name) {
        return $this->post[$name];
    }
    
    public function unsetPost($name, $flag=NULL) {
        if($flag===NULL) {
            unset($this->post[$name]);
        } else {
            unset($this->post);
            $this->post = array();
        }
        return $this;
    }
    
    
    
    public function _postArr() {
        return $this->post;
    }


    public function _RequestUri() {
        return $this->Request_Uri;
    }
    
    public function issetPost($name) {
        if(isset($this->post[$name])) {
            return true;
        } else {
            return false;
        }       
    }
    
    public function isEmpty($name) {
        if(empty($this->post[$name])) {
            return true;
        } else {
            return false;
        }
    }
    
    public function isArray($name) {
        if(is_array($this->post[$name])) {
            return true;
        } else {
            return false;
        }
    }
    
    public function __toString() {
        $class_vars = get_class_vars(get_class($this));
        
        $string = '';
        $string .= '<script type="text/javascript">
	$(document).ready(function() {
	var winheight = $("html").innerHeight();
 	$("div#debug-div").css("height", winheight+"px");
	});
	</script>';
        $string .= '<pre style="display:block; position:absolute; z-index:1000; border:6px solid #efefef; background:#fcfcfc; padding:22px; top:0px; left:0px;">';
        $string .= '<div style="display:block; width:600px; height:500px; overflow:auto; background:#fcfcfc; text-align:left; font-size:11px; font-family:helvetica, verdana, sans-serif; line-height:14px; color:#666666;" id="debug-div">';
           
        $string .= "Http: \n";
        foreach($class_vars as $key => $val) {
            $values = !is_object($this->{$key}) ? print_r($this->{$key}, true) : 'Object';
            $string .= $key.': '.$values."\n";
        }
        $string .= '</div></pre>';
        return $string;
    }
}