<?php

namespace lib\database\pdo;

/* 
 * aribu system - version 7.0.0
 * 
 * Copyright (C) 2018  Sascha Momcilovic
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 3 of the License, or (at your option) 
 * any later version. This program is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, see <http://www.gnu.org/licenses/>. 
 * 

 * 
 * @author: Sascha Momcilovic
 * @e-Mail: info@aribu.systems
 * @link:   http://aribu.systems
 * @date:   21.04.2018
 *  
 */

class DB {
    
    private $DBHost;
    private $DBName;
    private $DBUser;
    private $DBPassword;
    private $DBPort;
    private $pdo;
    public $ObjQuery;
    private $Connected = false;
    public $log;
    public $Success;
    public $RowCount   = 0;
    public $ColCount   = 0;
    public $QueryCount = 0;
    
    public function __construct($DBHost, $DBName, $DBUser, $DBPassword, $DBPort = 3306) {
        $this->log        = new Log();
        $this->DBHost     = $DBHost;
        $this->DBName     = $DBName;
        $this->DBUser     = $DBUser;
        $this->DBPassword = $DBPassword;
        $this->DBPort	  = $DBPort;
        $this->ConnectDB();
    }
    
    private function ConnectDB() {
        try {
            $DBOptions = array(\PDO::MYSQL_ATTR_INIT_COMMAND => "SET NAMES utf8", \PDO::ATTR_EMULATE_PREPARES => false, \PDO::ATTR_ERRMODE => \PDO::ERRMODE_EXCEPTION, \PDO::MYSQL_ATTR_USE_BUFFERED_QUERY => true);
            
            $this->pdo = new \PDO('mysql:dbname='.$this->DBName.';host='.$this->DBHost.';port='.$this->DBPort.';charset=utf8;', $this->DBUser, $this->DBPassword, $DBOptions);
            $this->Connected = true;
        } catch (\PDOException $ex) {
            echo $this->ExceptionLog($ex->getMessage());
            exit();
        }
    }
    
    public function Query($Statement, $Parameter) {
        if(!$this->Connected) {
            $this->ConnectDB();
        }
        try {
            $this->ObjQuery = $this->pdo->prepare($Statement);
            
            if(is_array($Parameter)) {
                if(!empty($Parameter)) {
                    foreach($Parameter as $key => $val) {
                        $this->Bind(":".$key, $val, null);
                    }
                }
            }
            
            $this->Success = $this->ObjQuery->execute();
            
            $this->QueryCount++;
            
        } catch (\PDOException $ex) {
            echo $this->ExceptionLog($ex->getMessage(), $Statement, $Parameter);
            exit();
        }
        return $this->Success;
    }
    
    public function Bind($param, $value, $type=null) {
        if(is_null($type)){
            switch (true){
                case is_int($value):
                  $type = \PDO::PARAM_INT;
                  break;
                case is_bool($value):
                    $type = \PDO::PARAM_BOOL;
                    break;
                case is_null($value):
                    $type = \PDO::PARAM_NULL;
                    break;
                default:
                    $type = \PDO::PARAM_STR;
            }
        }
        $this->ObjQuery->bindParam($param, $value, $type);
    }
    
    public function CountRows() {
        return $this->ObjQuery->rowCount();
    }
    
    public function lastInsertId() {
        return $this->pdo->lastInsertId();
    }	
        
    private function ExceptionLog($message, $sql = "", $parameter=array()) {
        $exception = 'Unhandled Exception. <br />';
        $exception .= $message;
        $exception .= "<br /> You can find the error back in the log.";
        $parameter_str = print_r($parameter, true);
        if (!empty($sql)) {
            $message .= "\r\nRaw SQL : " . $sql . "\n".$parameter_str;
        }
        $this->log->write($message, $this->DBName);

        return $exception;
    }
    
    public function __toString() {
        return $this;
    }
    
}