<?php

namespace lib\mailer;

/* 
 * aribus systems - Version 7.0.0
 * 
 * Copyright (C) 2018  Sascha Momcilovic
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 3 of the License, or (at your option) 
 * any later version. This program is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, see <http://www.gnu.org/licenses/>. 
 * 

 * 
 * @author: Sascha Momcilovic
 * @e-Mail: info@aribu.systems
 * @link:   https://aribu.systems
 * @date:   09.04.2015 
 */

class mailbuilder {
    
    /*
     * TT tiny template engine
     * @var object
     */
    public $TT;
    
    /*
     * mail
     * Mailer Class 
     * @var object
     */
    public $mail;
    
    /*
     * MailData
     * @var array
     */
    private $MailData   = array();
    
    /*
     * MailTemplate
     * @var string
     */
    private $MailTemplate;
    
    
    
    public function __construct($mail, $TT) {
        $this->mail = $mail;
        $this->TT   = $TT;
    }
    
    public function setFrom($param) {
        $this->mail->From = $param;
        return $this;
    }
            
    public function setFromName($param) {
        $this->mail->FromName = $param;
        return $this;
    }
    
    public function setSubject($param) {
        $this->mail->Subject = $param;
        return $this;
    }
    
    public function setMailer($param) {
        $this->mail->Mailer = $param;
        return $this;
    }
    
    public function setHostname($param) {
        $this->mail->Hostname = $param;
        return $this;
    }
    
    public function setCharSet($param) {
        $this->mail->CharSet = $param;
        return $this;
    }
    
    public function setSMTPHost($param) {
        $this->mail->Host = $param;
        return $this;
    }
    
    public function setSMTPKeepAlive($param) {
        $this->mail->SMTPKeepAlive = $param;
        return $this;
    }
       
    public function setSMTPUsername($param) {
        $this->mail->Username = $param;
        return $this;
    }
    
    public function setSMTPPassword($param) {
        $this->mail->Password = $param;
        return $this;
    }
    
    public function setSMTPPort($param) {
        $this->mail->Port = $param;
        return $this;
    }
    
    public function setAttachment($path, $name = "", $encoding = "base64", $type = "application/octet-stream") {
        $this->mail->AddAttachment($path, $name, $encoding, $type);
        return $this;
    }
    
    public function clearAttachments() {
        $this->mail->ClearAttachments();
        return $this;
    }
    
    public function clearAllRecipients() {
        $this->mail->ClearAllRecipients();
        return $this;
    }
    
    public function setDataArr(array $arr) {
        $this->MailData = $arr;
        return $this;
    }
    
    public function setData($name, $value) {
        $this->MailData[$name] = $value;
        return $this;
    }
    
    public function setMailTemplate($path) {
        $this->MailTemplate = $path;
        return $this;
    }
    
    public function setAddress($MailAddress, $mailAddressName) {
        $this->mail->AddAddress($MailAddress, $mailAddressName);
        return $this;
    }
    
    public function setBBCAddress($MailAddress, $mailAddressName) {
        $this->mail->AddBCC($MailAddress, $mailAddressName);
        return $this;
    }
    
    public function prepareData() {
        if(is_array($this->MailData)) {
            foreach($this->MailData as $key => $value) {
                $this->TT->BlockMarker[$key] = $value === '' || empty($value) || $value === null ? 0 : 1;
            }
        }
    }
    
    public function setBlockMarker($name, $value) {
        $this->TT->BlockMarker[$name] = $value;
        return $this;
    }
    
    public function sendEMail($flag = NULL, $Dyn = 0, $Dyn_Blocks=0, $EMail_Body='') {
        $this->prepareData();
        $this->TT->setMCArr($this->MailData);
        $Body    = $EMail_Body == '' ? \lib\Functions::replaceImageSRCToHost(\lib\Functions::replaceLinksforEMail($this->TT->getFilledTemplate($this->MailTemplate, $flag, $Dyn, $Dyn_Blocks))) : $EMail_Body;
        
        if( mb_detect_encoding($Body, 'UTF-8', true) === false ) {
            $Body_UTF8 = utf8_encode($Body);
        } else {
            $Body_UTF8 = $Body;
        }
        
        $AltBody = str_replace("<br />", "\n", str_replace("<br>", "\n", str_replace("<br/>", "\n",$Body_UTF8)));
        $AltBody = strip_tags($AltBody);
				
	$this->mail->Body    = utf8_decode($Body_UTF8);
	$this->mail->AltBody = utf8_decode($AltBody);
        
        if(!$this->mail->Send()) {
            # Error
            $return = $this->mail->ErrorInfo;
        } else {
            $return = true;
        }

        $this->mail->ClearAddresses();
        $this->TT->resetAll();
        
        return $return;
    }
}