<?php

namespace lib\session\backend;

/* 
 * aribu system - Version 7.0.0
 * 
 * Copyright (C) 2018  Sascha Momcilovic
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 3 of the License, or (at your option) 
 * any later version. This program is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, see <http://www.gnu.org/licenses/>. 
 * 

 * 
 * @author: Sascha Momcilovic
 * @e-Mail: info@aribu.systems
 * @link:   https://aribu.systems
 * @date:   21.04.2018
 *  
 */

class Session extends \lib\objects\Loader {

    public $setCookie = 0;
    public $SessionID;
    public $CookieName = 'dotCookie';
    public $BECookieName = 'dotBECookie';
    public $CookieImpCode = '00dOt07';


    public function __construct() {
        parent::__construct();
        session_start();
    }
    
    /**
     * Init one BE-Session-ID
     */
    function initBESession($beuserid = '', $beuserdata = '') {
      
        
        if($this->http->issetPost('login') === true) {
            if ($beuserid != '' && $beuserdata != '') {
                $this->deleteSession($beuserid, 'be_');
            }
        }
        
        # Caches the Session-ID & set cookie
        if ($beuserid != '' && $beuserdata != '') {

            $userDate = unserialize($beuserdata);
            $userCookieData[0] = $beuserid;
            $userCookieData[1] = $userData['sess_username'];
            $userCookieData[2] = $userData['BE_Rights'];
            $userCookieData[3] = $this->SessionID;
            $cookieString = implode($this->CookieImpCode, $userCookieData);
            setcookie($this->BECookieName, $cookieString, 0, "/");

            # Session Variable speichern
            $this->db->TablePreName = 'be_';
            
            $session_date = date('Y-m-d H:i:s', CURRENT_TIME);
            $this->db->Insert('session')
                     ->Parameter(array('session_id'         => $this->SessionID, 
                                       'session_user_id'    => $beuserid, 
                                       'session_date'       => $session_date, 
                                       'session_data'       => $beuserdata, 
                                       'session_deleted'    => 0))
                     ->doInsert();
            
            $this->db->Update('user')
                     ->WhereColName('uid')
                     ->WhereColValue($beuserid)
                     ->Set(array('last_login' => time()))
                     ->doUpdate();
        }


        if (isset($_COOKIE[$this->BECookieName])) {
            if (strstr($_COOKIE[$this->BECookieName], $this->CookieImpCode)) {
                $cookie_array = explode($this->CookieImpCode, $_COOKIE[$this->BECookieName]);
                $_SESSION['sess_id'] = $cookie_array[3];
            }
            return true;
        } else {
            return false;
        }
    }

    /**
     * Handles a BE session and delete old sessions from db->
     */
    function handleBESession($session_id, $TablePreName = "be_") {
        
        $this->db->TablePreName = $TablePreName;

        $this->db->Delete('session')
                 ->noWhere(1)
                 ->Andd("WHERE DATE_SUB(NOW(), INTERVAL 8 HOUR) > session_date AND session_deleted = 1")
                 ->doDelete();
        $this->db->Update('session')
                 ->noWhere(1)
                 ->Andd("WHERE DATE_SUB(NOW(), INTERVAL 7.59 HOUR) > session_date")
                 ->Parameter(array())
                 ->Set(array('session_deleted' => 1))
                 ->doUpdate();

        if (isset($_COOKIE[$this->BECookieName])) {
            if (strstr($_COOKIE[$this->BECookieName], $this->CookieImpCode)) {
                $cookie_array = explode($this->CookieImpCode, $_COOKIE[$this->BECookieName]);
                $_SESSION['sess_id'] = $cookie_array[3];
            }
        }
        $session_id = $session_id == '' && $_SESSION['sess_id'] != '' ? $_SESSION['sess_id'] : $session_id;

        $this->db->Select('*')
                 ->From('session')
                 ->WhereColName('session_id')
                 ->WhereColValue($session_id)
                 ->doSelect();
        if (!$this->db->Rows) {
            setcookie($this->BECookieName, '', time() - 3600, "/");
            @session_destroy();
            unset($_SESSION);
            return false;
        } else {
            if ($TablePreName == 'be_') {
                if (!empty($this->db->SelectPuffer['session_data'])) {
                    $sess_userdata = unserialize($this->db->SelectPuffer['session_data']);
                    $_SESSION['session_id'] = $session_id;
                    foreach ($sess_userdata as $key => $val) {
                        $_SESSION["" . $key . ""] = $val;
                    }
                }
            }
            
            $this->db->Update('session')
                     ->WhereColName('session_id')
                     ->WhereColValue($session_id)
                     ->Set(array('session_date' => 'NOW()', 'session_deleted' => 0))
                     ->SetType(false)
                     ->doUpdate();
            return true;
        }
    }

    public function Update($session_id, $client_id) {
        $this->db->TablePreName = 'be_';
        foreach($_SESSION as $key => $val) {
            $_SESSION["" . $key . ""] = $val;
        }
        
        $be_userdata = serialize($_SESSION);
        
        $this->db->Update('session')
                 ->WhereColName('session_id')
                 ->WhereColValue($session_id)
                 ->Set(array('session_data' => $be_userdata))
                 ->doUpdate();
    }
    
    /**
     * Delete a login-session
     */
    public function deleteSession($session_user_id, $TablePreName) {  
        $this->db->TablePreName = $TablePreName;
        $this->db->Delete('session')
                 ->WhereColName('session_user_id')
                 ->WhereColValue($session_user_id)
                 ->doDelete();
        $this->db->Update('user')
                 ->WhereColName('uid')
                 ->WhereColValue($session_user_id)
                 ->Set(array('last_login' => time()))
                 ->Parameter(array())
                 ->doUpdate();
        setcookie($this->BECookieName, '', time() - 3600, "/");
        @session_destroy();
        unset($_SESSION);
    }
    
    public function __destruct() {
        session_destroy();
    }
    
}