<?php

namespace plugins\search\lib;

/* 
 * aribus systems - Version 7.0.0
 * 
 * Copyright (C) 2018  Sascha Momcilovic
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 3 of the License, or (at your option) 
 * any later version. This program is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, see <http://www.gnu.org/licenses/>. 
 * 
 * 
 * @author:  Sascha Momcilovic
 * @e-Mail:  info@it-orange.de
 * @link:    http://www.it-orange.de
 * @date:    02.04.2006 
 * @update:  16.07.2015
 */

class Search {
    
    /**
     * DFE object
     * @var object
     */
    private $DFE;
    
    public $Limit = 10;
    public $Page = 0;
    public $SearchString = false;
    
    public $TemplatePage = '';
    public $LanguageFolder = '';
    public $Lang = '';
    
    public $UnionSelect = '';

    public function __construct($DFE) {
        $this->DFE = $DFE;
        
        if ($this->DFE->http->issetPost('SEITE') === true && $this->DFE->http->isEmpty('SEITE') === false) {
            $page = intval($this->DFE->http->_post('SEITE')) <= 0 ? 1 : intval($this->DFE->http->_post('SEITE'));
            $this->Page = ($page - 1) * $this->Limit;
        } else {
            $this->DFE->http->unsetPost('SEITE');
        }

        $this->DFE->IndexFollow = 0;
        $this->SearchString     = \lib\Functions::removeCriticSigns(strip_tags(trim($this->DFE->http->_post('search_string'))));
        $this->LanguageFolder   = SYS_LANGUAGE != $GLOBALS['LANGUAGE'] ? $GLOBALS['LANGUAGE'].'/' : '';
        $this->Lang             = SYS_LANGUAGE != $GLOBALS['LANGUAGE'] ? '/'.$GLOBALS['LANGUAGE'] : '';
    }
    
    public function SearchDB() {
        $search_string_wcount = str_word_count($this->SearchString);
        $match_string = $search_string_wcount == 1 ? htmlentities(\dotlib\dotFunc::removeCriticSigns(strip_tags(trim(utf8_decode($this->SearchString))))) . "*" : "\"" . htmlentities(\dotlib\dotFunc::removeCriticSigns(strip_tags(trim(utf8_decode($this->SearchString))))) . "\"";
        $match_string_utf8 = $search_string_wcount > 1 ? strip_tags(trim($this->SearchString)) . "*" : "\"" . strip_tags(trim($this->SearchString)) . "\"";
        $this->DFE->db->Select("DISTINCT MATCH (c.bodytext) AGAINST ('" . $match_string . "' IN BOOLEAN MODE) AS score, c.bodytext, c.headline, p.uid, p.nav_title, 0 as type")
                      ->From("FROM fe_content c, fe_" . $this->DFE->Pages . " p")
                      ->noWhere(1)
                      ->Andd("WHERE (MATCH (c.bodytext) AGAINST ('" . $match_string . "' IN BOOLEAN MODE) OR MATCH (p.keywords) AGAINST ('" . $match_string . "' IN BOOLEAN MODE)) "
                              . "AND c.lang = :clang "
                              . "AND (c.CType = 'HTML' OR c.CType = 'TXT' OR c.CType = 'TXTI' OR c.CType = 'TEXT') "
                              . "AND c.pid = p.uid ".str_replace("%%%n%%%", "p.", $this->DFE->and_lang)." "
                              . "AND p.hidden != 1 "
                              . "AND p.docType = 64 "
                              . "AND p.search = 1 "
                              . "AND (SELECT pp.nav_title FROM fe_" . $this->DFE->Pages . " np, fe_" . $this->DFE->Pages . " pp WHERE np.lft BETWEEN pp.lft AND pp.rgt AND np.uid = p.uid AND pp.nav_title LIKE '%".URL_HOST."%') = '".URL_HOST."' ")
                      ->Group("GROUP BY p.uid")
                      ->Order("ORDER BY score DESC")
                      ->Union($this->UnionSelect)
                      ->Parameter($this->DFE->and_lang != '' ? array('lang' => $GLOBALS['LANGUAGE'], 'clang' => $GLOBALS['LANGUAGE']) : array('clang' => $GLOBALS['LANGUAGE']))
                      ->SelectMethod(1)
                      ->doSelect(true);
        $num = $this->DFE->db->Rows;
        
        $this->DFE->SelectMethod(1)
                  ->Parameter(array('lang' => $GLOBALS['LANGUAGE']))
                  ->doCachedSelect("Limit " . $this->Page . ", " . $this->Limit);
        
        return $num;

    }
    
    public function getSearchResults() {
        if (isset($this->SearchString) && !empty($this->SearchString) && trim($this->SearchString) != '' && ($this->DFE->http->issetPost('in') === false || $this->DFE->http->_post('in') == 'Pages')) {
            
            # Template
            $this->TemplatePage = "/_templates/".$GLOBALS['CLIENT_TEMPLATE_FOLDER']."plugins/search/" . $this->LanguageFolder . "search.html";
            
            $num = $this->SearchDB();
            if ($num) {
                
                $i = 0;
                foreach ($this->DFE->db->SelectPuffer as $key => $row) {
                    $google = "";

                    if ($row['type'] == 0) {
                        $linkpath = \lib\fe_menu::getTreePathNames($row['uid'], '', 0, 1, 1, '', ' AND p.docType != 0 ');
                        $navTitle = $row['nav_title'];
                        $headline = '<a href="' . $linkpath . '" class="search-head">' . $navTitle . '</a>';
                        $link = '<a href="' . $linkpath . '" class="search-head top">' . URL_PROTOCOL . '://' . URL_HOST . $this->Lang . $linkpath . '</a>';
                    } 
                    
                    $bdtext = $row['bodytext'];

                    # Javascripte entfernen
                    if (preg_match_all('/((<[\s\/]*script\b[^>]*>)([^>]*)(<\/script>))/ims', $bdtext, $bdtext_js)) {
                        foreach ($bdtext_js[0] as $skey => $sval) {
                            $bdtext = str_replace($sval, "", $bdtext);
                        }
                    }

                    # Links entfernen
                    if (preg_match_all('!<a.*href\="-?(\S+)"([^>]*)>.+</a>!siUu', $bdtext, $intern_links)) {
                        foreach ($intern_links[0] as $lkey => $lval) {
                            $bdtext = str_replace($lval, "", $bdtext);
                        }
                    }

                    $bdtext = strip_tags($bdtext);


                    $bdtext = str_replace("\n", " ", $bdtext);
                    $bdtext = str_replace("\t", " ", $bdtext);
                    $bdtext = str_replace(".", ". ", $bdtext);
                    $bdtext = str_replace(":", ": ", $bdtext);
                    $bdtext = str_replace("|", "", $bdtext);
                    preg_match_all('!\S+!m', $bdtext, $teaser);
                    $w = 1;
                    if (is_array($teaser[0])) {
                        foreach ($teaser[0] as $key => $val) {
                            if ($w < 25) {
                                $google .= \lib\Functions::mb_trim($val) . " ";
                            }
                            $w++;
                        }
                    }
                    
                    
                    $this->DFE->db->setMC($i, 'nr', $i+1)
                                  ->setMC($i, 'teaser', \lib\Functions::mb_trim($google))
                                  ->setMC($i, 'link',   $link)
                                  ->setMC($i, 'headline', $headline)
                                  ->setMC($i, 'linkpath',   $linkpath)
                                  ->setMC($i, 'navTitle', $navTitle);
                    $i++;
                }
            } else {
                $this->DFE->BlockMarker['result'] = 0;
            }
            
            $scrolling = \lib\Functions::scrolling($num, "search_string=" . \lib\Functions::removeCriticSigns(strip_tags(trim($this->SearchString))) . $union, $lang . $this->RequestURI . "?", $limit);
            
            if ($scrolling === false) {
                $this->DFE->TT->BlockMarker['scrolling'] = 0;
            } 
            $this->DFE->TT->setMC('num', $num);
        } else {
            $this->DFE->TT->BlockMarker['result'] = 0;
            $this->DFE->TT->setMC('num', '0');
        }
        
        return $this->DFE->TT->getFilledTemplate($this->TemplatePage, NULL, 1);
    }
}